#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import os
import cherrypy
import re
from cocktail.controllers import serve_file, resolve_object_ref
from woost.models import Item, get_current_user, RenderPermission
from woost.models.rendering import (
    require_rendering,
    ImageFactory,
    BadRenderingRequest
)
from woost.models.rendering.formats import formats_by_extension
from woost.controllers.basecmscontroller import BaseCMSController

generic_image_factory_pattern = re.compile(r"^factory(\d+)$")


class ImagesController(BaseCMSController):
    """A controller that produces, caches and serves images representing the
    different kinds of content managed by the CMS.
    """

    def __call__(self, id, processing, *args, **kwargs):

        # Get the requested element
        item = resolve_object_ref(Item, id)

        # Make sure the selected element exists
        if item is None:
            raise cherrypy.NotFound()

        # Normalize the URL to use a local id
        if id == item.global_id:
            raise cherrypy.HTTPRedirect(
                cherrypy.url().replace("/" + id + "/", "/%d/" % item.id)
            )

        # Handle legacy image requests (woost < 0.8)
        if args or kwargs or "(" in processing:
            raise cherrypy.HTTPError(410)

        # Parse the given processing string, splitting the image factory from
        # the image format (ie. "home_thumbnail.png" -> ("home_thumbnail", "PNG"))
        parts = processing.rsplit(".", 1)
        parameters = None

        if len(parts) == 2:
            factory_id, ext = parts
            format = formats_by_extension.get(ext)
            if format is None:
                raise cherrypy.HTTPError(
                    400,
                    "Invalid image extension: %s" % ext
                )
        else:
            factory_id = processing
            format = None

        factory = ImageFactory.get_instance(identifier = factory_id)

        if factory is None:
            match = generic_image_factory_pattern.match(factory_id)
            if match:
                try:
                    factory_id = int(match.group(1))
                    factory = ImageFactory.require_instance(factory_id)
                except:
                    pass

            if factory is None:
                raise cherrypy.HTTPError(
                    400,
                    "Invalid image factory id: %s" % factory_id
                )

        # Deny access to unauthorized elements
        get_current_user().require_permission(
            RenderPermission,
            target = item,
            image_factory = factory
        )

        try:
            image_cache_file = require_rendering(
                item,
                factory,
                format,
                parameters
            )
        except BadRenderingRequest, ex:
            raise cherrypy.HTTPError(400, ex.message)

        return serve_file(image_cache_file)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .file import File
from .block import Block


class FlashBlock(Block):

    instantiable = True
    view_class = "cocktail.html.SWFObject"

    members_order = [
        "swf_file",
        "width",
        "height",
        "flash_version",
        "flash_vars",
        "flash_params",
        "flash_attributes"
    ]

    swf_file = schema.Reference(
        type = File,
        rquired = True,
        relation_constraints = {"mime_type": "application/x-shockwave-flash"},
        related_end = schema.Collection(),
        member_group = "content"
    )

    width = schema.Integer(
        required = True,
        min = 0,
        member_group = "content"
    )

    height = schema.Integer(
        required = True,
        min = 0,
        member_group = "content"
    )

    flash_version = schema.String(
        required = True,
        default = "9.0.0",
        member_group = "content"
    )

    flash_vars = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "content"
    )

    flash_params = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "content"
    )

    flash_attributes = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.flash_file = self.swf_file.get_uri()
        view.flash_width = self.width
        view.flash_height = self.height
        view.flash_version = self.flash_version
        view.flash_vars = self._split_config(self.flash_vars)
        view.flash_params = self._split_config(self.flash_params)
        view.flash_attributes = self._split_config(self.flash_attributes)

    def _split_config(self, config_text):

        config = {}

        if config_text:
            for line in config_text.split("\n"):
                try:
                    pos = line.find("=")
                    key = line[:pos]
                    value = line[pos + 1:]
                except:
                    pass
                else:
                    config[key.strip()] = value.strip()

        return config




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.surveys.survey import Survey


class SurveyResultsAction(UserAction):

    min = 1
    max = 1
    content_type = Survey
    included = frozenset(["toolbar", "item_buttons"])


SurveyResultsAction("survey_results").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from woost.models import type_groups, block_type_groups, TypeGroup


type_groups["setup"].append(TypeGroup("ecommerce"))

block_type_groups.append(TypeGroup("blocks.ecommerce"))



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from PIL import Image
from tempfile import mkdtemp
from shutil import rmtree
from subprocess import Popen
from cocktail import schema
from woost.models.publishable import Publishable
from woost.models.uri import URI
from woost.models.user import User
from woost.models.rendering.renderer import Renderer


class HTMLRenderer(Renderer):
    """A content renderer that handles XHTML/HTML pages."""

    instantiable = True

    mime_types = set([
        "text/html",
        "text/xhtml",
        "application/xhtml"
    ])

    command = schema.String(
        required = True,
        default = "python -m woost.models.rendering.renderurl "
                  "%(source)s %(dest)s"
    )

    window_width = schema.Integer(
        min = 0
    )

    window_height = schema.Integer(
        min = 0
    )

    external_urls_only = schema.Boolean(
        required = True,
        default = True
    )

    def can_render(self, item, **parameters):
        cls = self.external_urls_only and URI or Publishable
        return (
            isinstance(item, cls)
            and item.mime_type in self.mime_types
            and item.is_accessible(
                user = User.get_instance(qname = "woost.anonymous_user")
            )
        )

    def render(self, item,
        window_width = None,
        window_height = None,
        **parameters
    ):
        temp_path = mkdtemp()

        try:
            temp_image_file = os.path.join(temp_path, "thumbnail.png")

            command = self.command % {
                "source": item.get_uri(host = "."),
                "dest": temp_image_file
            }

            if window_width is None:
                window_width = self.window_width

            if window_width is not None:
                command += " --min-width %d" % window_width

            if window_height is None:
                window_height = self.window_height

            if window_height is not None:
                command += " --min-height %d" % window_height

            Popen(command, shell = True).wait()
            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import Configuration

Configuration.add_member(
    schema.String("external_files_host",
        text_search = False,
        member_group = "publication",
        listed_by_default = False
    )
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from tpv.currencies import currency_alpha
from cocktail.modeling import OrderedDict
from cocktail.events import Event
from cocktail.translations import translations, get_language
from cocktail import schema
from cocktail.controllers import Location
from woost.models import Item, get_current_website


class PaymentGateway(Item):

    instantiable = False
    visible_from_root = False
    payment_gateway_controller_class = \
        "woost.extensions.payments.paymentgatewaycontroller.PaymentGatewayController"

    transaction_notified = Event(
        """An event triggered when the payment gateway notifies the application
        of the outcome of a payment transaction.

        @param payment: The payment that the notification is sent for.
        @type payment: L{Payment<tpv.payment.Payment>}
        """
    )

    members_order = [
        "label",
        "test_mode",
        "currency"
    ]

    label = schema.String(
        required = True,
        translated = True
    )

    test_mode = schema.Boolean(
        required = True,
        default = True
    )

    currency = schema.String(
        required = True,
        enumeration = currency_alpha,
        translatable_enumeration = False,
        text_search = False
    )

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.name, language)

    def initiate_payment(self, payment_id):
        """Begin a payment transaction, redirecting the user to the payment
        gateway.

        @param payment_id: The identifier of the payment to execute.
        """
        url, params = self.get_payment_form_data(payment_id, get_language())

        location = Location(url)
        location.method = "POST"
        location.form_data = OrderedDict(params)
        location.go()

    def get_payment_url(self, *args, **kwargs):

        website = get_current_website()
        location = Location()
        location.relative = False

        if website.https_policy != "never":
            location.scheme = "https"

        location.host = website.hosts[0]
        location.path_info = "payments/" + str(self.id)
        location.join_path(*args)
        location.query_string.update(kwargs)

        return unicode(location)

    @property
    def handshake_url(self):
        return self.get_payment_url("handshake")

    @property
    def notification_url(self):
        return self.get_payment_url("notification")




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re
from woost.models import Configuration, get_current_website

html_expr = re.compile(r"</?[a-z][^>]*>")
li_expr = re.compile(r"<li(\s+[^>]*)?>")
br_expr = re.compile(r"<br\s*/?>")
block_end_expr = re.compile(r"</(p|li)>")
excessive_line_jumps_expr = re.compile(r"\n{2,}")

def export_content(content):
    if content:
        content = content.replace("\r", "\n")
        content = li_expr.sub("* ", content)
        content = br_expr.sub("\n", content)
        content = block_end_expr.sub("\n", content)
        content = html_expr.sub("", content)
        content = excessive_line_jumps_expr.sub("\n", content)
        content = content.strip("\n")
    return content

def get_publishable_website(publishable):
    if publishable.websites:
        if len(publishable.websites) == 1:
            return publishable.websites[0]
        else:
            current_website = get_current_website()
            if current_website in publishable.websites:
                return current_website

    config = Configuration.instance
    if len(config.websites) == 1:
        return config.websites[0]




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from urllib import urlencode
import cherrypy
from cocktail.modeling import getter
from cocktail.events import event_handler
from cocktail.controllers import Controller, request_property
from cocktail.html import renderers, templates
from woost import app


class BaseCMSController(Controller):
    """Base class for all CMS controllers."""

    @request_property
    def view(self):
        if self.view_class:
            view = templates.new(self.view_class)
            cms = self.context["cms"]
            output = self.output
            cms.producing_output(controller = self, output = output)

            for key, value in output.iteritems():
                setattr(view, key, value)

            view.submitted = self.submitted
            view.successful = self.successful
            return view

    def _render_template(self):

        renderer = None
        format = self.rendering_format

        if format:
            if format == "html":
                format = "default"
            try:
                renderer = getattr(renderers, "%s_renderer" % format)
            except AttributeError:
                raise ValueError("Can't render '%s' using format '%s'"
                    % (template, format))

        if self.render_as_fragment:
            return self.view.render(renderer = renderer)
        else:
            return self.view.render_page(renderer = renderer)

    def application_uri(self, *args, **kwargs):
        """Builds an absolute URI from a set of path components and parameters.

        @param args: A set of path components, relative to the application
            root. Takes any object that can be expressed as an unicode string.

        @param kwargs: A set of query string parameters to be included on the
            generated URI.

        @return: The generated absolute URI.
        @rtype: unicode
        """
        path = (unicode(arg).strip("/") for arg in args)
        uri = (
            self.context["cms"].virtual_path
            + u"/".join(component for component in path if component)
        )

        if kwargs:
            uri += "?" + urlencode(
                dict(
                    (key, value)
                    for key, value in kwargs.iteritems()
                    if not value is None
                ),
                True
            )

        # Contextual URI prefix
        context_prefix = self.context.get("uri_prefix")

        if context_prefix:
            uri = context_prefix.rstrip("/") + "/" + uri.lstrip("/")

        if uri.startswith("./"):
            uri = uri[2:]

        return uri

    def contextual_uri(self, *args, **kwargs):
        """Builds an URI relative to the currently requested publishable item.

        @param args: A set of path components that will be appended to the
            publishable's URI. Accepts any object that can be expressed as an
            unicode string.

        @param kwargs: A set of query string parameters to be included on the
            generated URI.

        @return: The generated absolute URI.
        @rtype: unicode
        """
        publishable = self.context["publishable"]
        uri = app.url_resolver.get_path(publishable)

        if uri is None:
            return None

        return self.application_uri(
            uri,
            *args,
            **kwargs
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re
from decimal import Decimal
from fractions import Fraction
from simplejson import dumps, loads
import urllib2
import datetime
import hashlib
import base64
from BTrees.OOBTree import OOBTree
from cocktail.events import when
from cocktail.modeling import (
    ListWrapper,
    SetWrapper,
    DictWrapper
)
from cocktail.pkgutils import get_full_name, resolve
from cocktail import schema
from cocktail.persistence import datastore
from woost.models.item import Item

RegExp = type(re.compile(""))


class Synchronization(object):

    date_format = "%Y-%m-%d"
    time_format = "%H:%M:%S"
    datetime_format = date_format + " " + time_format

    def __init__(self,
        remote_installation_url = None,
        remote_user = None,
        remote_password = None
    ):
        self.__local_copies = {}
        self.remote_installation_url = remote_installation_url
        self.remote_user = remote_user
        self.remote_password = remote_password

    def get_local_copy(self, global_id):
        return (
            self.__local_copies.get(global_id)
            or Item.get_instance(global_id = global_id)
        )

    def process_manifest(self):
        manifest = get_manifest()
        for global_id, object_hash in manifest.iteritems():
            if object_hash is None:
                obj = Item.require_instance(global_id = global_id)
                object_hash = self.get_object_state_hash(obj)
                manifest[global_id] = object_hash
            yield global_id, object_hash

    def get_object_state_hash(self, obj):
        state = self.export_object_state(obj)
        state = self.__stabilize(state)

        try:
            json = dumps(state)
        except Exception, error:
            raise ValueError("Can't serialize %r (%s)" % (
                obj,
                error
            ))

        return hashlib.md5(json).hexdigest()

    def __stabilize(self, data):
        if isinstance(data, dict):
            return [
                [key, self.__stabilize(data[key])]
                for key in sorted(data)
            ]
        elif isinstance(data, list):
            return [self.__stabilize(value) for value in data]
        else:
            return data

    def serialize_object_state(self, obj):
        return dumps(self.export_object_state(obj))

    def export_object_state(self, obj):

        state = {
            "__class__": obj.__class__.full_name
        }

        for key, member in obj.__class__.members().iteritems():
            if member.synchronizable:
                value = self.export_member(obj, member)
                state[key] = value

        return state

    def export_member(self, obj, member):
        if member.translated:
            return dict(
                (lang, self.export_value(obj.get(member, lang)))
                for lang in obj.translations
            )
        elif isinstance(member, schema.Collection):
            return [
                self.export_value(item)
                for item in (obj.get(member) or ())
            ]
        else:
            return self.export_value(obj.get(member))

    def export_value(self, value):
        if isinstance(value, Item):
            return value.global_id
        elif isinstance(value, datetime.datetime):
            return value.strftime(self.datetime_format)
        elif isinstance(value, datetime.date):
            return value.strftime(self.date_format)
        elif isinstance(value, datetime.time):
            return value.strftime(self.time_format)
        elif isinstance(value, Decimal):
            return str(value)
        elif isinstance(value, Fraction):
            return str(value)
        elif isinstance(value, tuple):
            return [self.export_value(item) for item in value]
        elif isinstance(value, (ListWrapper, SetWrapper, DictWrapper)):
            return self.export_value(value._items)
        elif isinstance(value, RegExp):
            return base64.b64encode(value.pattern)
        elif isinstance(value, schema.SchemaClass):
            return get_full_name(value)
        else:
            return value

    def import_object_state(self, obj, state):

        if isinstance(obj.__class__, schema.Schema):
            obj_schema = obj.__class__
        else:
            obj_schema = resolve(state["__class__"])

        for key, value in state.iteritems():

            if key == "__class__":
                continue

            member = obj_schema.get_member(key)
            if member is None:
                continue

            if member.synchronizable:
                self.import_member(obj, member, value)

    def import_member(self, obj, member, value):
        if member.translated:
            for lang, translated_value in value.iteritems():
                translated_value = self.import_value(member, translated_value)
                schema.set(obj, member, translated_value, lang)
        else:
            value = self.import_value(member, value)
            schema.set(obj, member, value)

    def import_value(self, member, value):

        if value is None:
            return None
        elif isinstance(member, schema.Reference):
            if member.class_family:
                return resolve(value)
            else:
                return self.get_local_copy(value)
        elif isinstance(member, schema.DateTime):
            return datetime.datetime.strptime(value, self.datetime_format)
        elif isinstance(member, schema.Date):
            return datetime.datetime.strptime(value, self.date_format).date()
        elif isinstance(member, schema.Time):
            return datetime.datetime.strptime(value, self.time_format).time()
        elif isinstance(member, schema.Decimal):
            return Decimal(value)
        elif isinstance(member, schema.Fraction):
            return Fraction(value)
        elif isinstance(member, schema.Tuple):
            return tuple(
                self.import_value(submember, subvalue)
                for submember, subvalue in zip(member.items, value)
            )
        elif isinstance(member, schema.Collection):
            return member.default_type([
                self.import_value(member.items, item)
                for item in value
            ])
        elif isinstance(member, schema.RegularExpression):
            return base64.b64decode(value)
        else:
            return value

    def _sync_request(self, action):

        url = (
            self.remote_installation_url.rstrip("/")
            + "/cms/synchronization/"
            + action
        )

        request = urllib2.Request(url)

        # Add authentication headers
        if self.remote_user:
            request.add_header("X-Woost-User", self.remote_user)

        if self.remote_password:
            request.add_header("X-Woost-Password", self.remote_password)

        return urllib2.urlopen(request)

    def download_file(self, file, chunk_size = 1024):
        response = self._sync_request("file/" + file.global_id)
        with open(file.file_path, "wb") as dest:
            while True:
                chunk = response.read(chunk_size)
                if not chunk:
                    break
                dest.write(chunk)

    def compare_manifests(self):

        response = self._sync_request("manifest")

        incomming = set()
        modified = set()
        local_manifest = get_manifest()

        for line in response:
            line = line.strip()
            pos = line.find(" ")
            global_id = line[:pos]
            remote_hash = line[pos + 1:].strip()
            local_obj = Item.get_instance(global_id = global_id)

            if local_obj is None:
                incomming.add(global_id)
            else:
                local_hash = local_manifest.get(global_id)
                if local_hash is None:
                    local_hash = self.get_object_state_hash(local_obj)
                    local_manifest[global_id] = local_hash
                if remote_hash != local_hash:
                    modified.add(global_id)

        return {"incomming": incomming, "modified": modified}

    def compare_content(self, establish_relations = False):

        self.__local_copies.clear()

        # Find incomming / outgoing / modified items
        manifest_diff = self.compare_manifests()

        # Retrieve the state for remotely added/modified objects
        remote_ids = (manifest_diff["incomming"]  | manifest_diff["modified"])
        changes = {}

        if remote_ids:
            response = self._sync_request("state/" + ",".join(remote_ids))
            remote_state = loads(response.read())

            # Build a local copy of each object that has been added by the remote
            # installation
            for global_id in manifest_diff["incomming"]:
                state = remote_state[global_id]
                model = resolve(state["__class__"])
                local_copy = model(id = None, global_id = global_id)
                local_copy.bidirectional = establish_relations
                self.__local_copies[global_id] = local_copy

            # Initialize the local copies with the received state
            for global_id in manifest_diff["incomming"]:
                self.import_object_state(
                    self.__local_copies[global_id],
                    remote_state[global_id]
                )

            for global_id in manifest_diff["modified"]:
                local_copy = Item.require_instance(global_id = global_id)
                item_remote_state = {}
                self.import_object_state(
                    item_remote_state,
                    remote_state[global_id],
                )

                changes[global_id] = {
                    "local": local_copy,
                    "remote": item_remote_state,
                    "diff": list(
                        (member, lang)
                        for member, lang in schema.diff(
                            local_copy,
                            item_remote_state,
                            schema = local_copy.__class__
                        )
                        if member.synchronizable
                    )
                }


        return {
            "incomming": self.__local_copies.values(),
            "modified": changes
        }

MANIFEST_KEY = "woost.manifest"
debug = False

def get_manifest():
    manifest = datastore.root.get(MANIFEST_KEY)

    if manifest is None:
        datastore.root[MANIFEST_KEY] = manifest = OOBTree()

    return manifest

def rebuild_manifest(eager = True):
    datastore.root[MANIFEST_KEY] = manifest = OOBTree()

    if eager:
        sync = Synchronization()

    for obj in Item.select(Item.synchronizable.equal(True)):
        global_id = obj.global_id
        if global_id:
            if eager:
                object_hash = sync.get_object_state_hash(obj)
                manifest[global_id] = object_hash
            else:
                manifest[global_id] = None

@when(Item.inserting)
def _insert_into_manifest(e):
    obj = e.source
    if obj.synchronizable:
        global_id = e.source.global_id
        if global_id:
            get_manifest()[global_id] = None
            if debug:
                print styled("Manifest: declared " + obj.global_id, "bright_green")

@when(Item.changed)
def _invalidate_manifest(e):

    from cocktail.styled import styled

    obj = e.source

    if not obj.is_inserted:
        return

    # Elegibility for synchronization changed
    if e.member is Item.synchronizable:

        # Object becomes synchronizable: add it to the manifest
        if e.value:
            if obj.global_id:
                get_manifest()[obj.global_id] = None
                if debug:
                    print styled("Manifest: declared " + obj.global_id, "bright_green")
        # Object ceases to be synchronizable: remove it from the manifest
        else:
            get_manifest().pop(obj.global_id, None)
            if debug:
                print styled("Manifest: removed " + obj.global_id, "magenta")

        return

    elif not obj.synchronizable:
        return

    # Object global identifier changed
    if e.member is Item.global_id:
        manifest = get_manifest()

        if e.previous_value:
            manifest.pop(e.previous_value, None)
            if debug:
                print styled("Manifest: removed " + obj.global_id, "magenta")

        if e.value:
            manifest[e.value] = None
            if debug:
                print styled("Manifest: declared " + obj.global_id, "bright_green")

        return

    elif not obj.global_id:
        return

    # Changing a synchronizable member: invalidate the stored object's hash in
    # the manifest, if any
    if e.member.synchronizable:
        get_manifest()[obj.global_id] = None
        if debug:
            print styled("Manifest: invalidated " + obj.global_id, "cyan")

@when(Item.deleted)
def _remove_from_manifest(e):
    get_manifest().pop(e.source.global_id, None)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import request_property, get_parameter
from woost.models import Item
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.models import Block
from woost.models.blockutils import add_block
from woost.controllers.backoffice.editblockscontroller \
    import get_slot_parameter


class DropBlockController(BaseBackOfficeController):

    @request_property
    def block(self):
        return get_parameter(schema.Reference("block", type = Block))

    @request_property
    def source_parent(self):
        return get_parameter(schema.Reference("source_parent", type = Item))

    @request_property
    def source_slot(self):
        return get_slot_parameter(self.source_parent, "source_slot")

    @request_property
    def target_parent(self):
        return get_parameter(schema.Reference("target_parent", type = Item))

    @request_property
    def target_slot(self):
        return get_slot_parameter(self.target_parent, "target_slot")

    @request_property
    def anchor(self):
        return get_parameter(schema.Reference("anchor", type = Block))

    def __call__(self, **kwargs):

        block = self.block

        if block is None:
            raise cherrypy.HTTPError(400,
                "Must supply a 'block' parameter indicating the block to "
                "relocate"
            )

        source_parent = self.source_parent

        if source_parent is None:
            raise cherrypy.HTTPError(400,
                "Must supply a 'source_parent' parameter indicating the "
                "container that the block is being moved from"
            )

        source_slot = self.source_slot

        if source_slot is None:
            raise cherrypy.HTTPError(400,
                "Must supply a 'source_slot' parameter indicating the "
                "reference or collection that the block is being removed from"
            )

        target_parent = self.target_parent

        if target_parent is None:
            raise cherrypy.HTTPError(400,
                "Must supply a 'target_parent' parameter indicating the "
                "container that the block is being moved from"
            )

        target_slot = self.target_slot

        if target_slot is None:
            raise cherrypy.HTTPError(400,
                "Must supply a 'target_slot' parameter indicating the "
                "reference or collection that the block is being removed from"
            )

        anchor = self.anchor

        # Remove the block from its current position
        if isinstance(source_slot, schema.Reference):
            source_parent.set(source_slot, None)
        else:
            collection = source_parent.get(source_slot)
            schema.remove(collection, block)

        # Add it to its new location
        add_block(
            block,
            target_parent,
            target_slot,
            positioning = "append" if anchor is None else "before",
            anchor = anchor
        )

        datastore.commit()
        return ""




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .slot import Slot
from .block import Block
from .elementtype import ElementType


class ContainerBlock(Block):

    instantiable = True
    type_group = "blocks.layout"
    view_class = "woost.views.ContainerBlockView"

    members_order = [
        "element_type",
        "list_type"
    ]

    element_type = ElementType(
        member_group = "content"
    )

    list_type = schema.String(
        required = True,
        default = "div",
        enumeration = [
            "div",
            "ul",
            "ol",
            "dl"
        ],
        member_group = "content"
    )

    blocks = Slot()

    def init_view(self, view):
        Block.init_view(self, view)

        if self.element_type == "dd":
            view.tag = None
            view.blocks_list.tag = "dd"
        else:
            view.tag = self.element_type

        view.blocks_list.tag = self.list_type
        view.blocks = self.blocks
        return view

    def get_block_proxy(self, view):
        if self.element_type == "dd":
            return view.blocks_list
        return view




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import User


User.add_member(
    schema.Boolean(
        "confirmed_email",
        default = False,
        required = True
    )
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from simplejson import dumps
from cocktail.translations import translations, get_language
from cocktail.events import Event
from cocktail.controllers import context
from woost.models import (
    Extension,
    Configuration,
    get_current_user
)

translations.define("GoogleAnalyticsExtension",
    ca = u"Google Analytics",
    es = u"Google Analytics",
    en = u"Google Analytics"
)

translations.define("GoogleAnalyticsExtension-plural",
    ca = u"Google Analytics",
    es = u"Google Analytics",
    en = u"Google Analytics"
)

translations.define("GoogleAnalyticsExtension.account",
    ca = u"Compte",
    es = u"Cuenta",
    en = u"Account"
)


class GoogleAnalyticsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Integra el lloc web amb Google Analytics.""",
            "ca"
        )
        self.set("description",
            u"""Integra el sitio web con Google Analytics.""",
            "es"
        )
        self.set("description",
            u"""Integrates the site with Google Analytics.""",
            "en"
        )

    def _load(self):

        from woost.extensions.googleanalytics import (
            strings,
            configuration,
            website,
            document,
            eventredirection
        )

        from cocktail.events import when
        from woost.controllers import CMSController

        @when(CMSController.producing_output)
        def handle_producing_output(e):
            publishable = e.output.get("publishable")
            if (
                publishable is None
                or getattr(publishable, "is_ga_tracking_enabled", lambda: True)()
            ):
                html = e.output.get("head_end_html", "")
                if html:
                    html += " "
                html += self.get_analytics_page_hit_script(publishable)
                e.output["head_end_html"] = html

    inclusion_code = {
        "ga.js-async":
            """
            <script type="text/javascript">
                var _gaq = _gaq || [];
                %(create_tracker_command)s
                %(commands)s
                (function() {
                var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
                ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
                var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
                })();
            </script>
            """,
        "ga.js-sync":
            """
            <script type="text/javascript" src="http://www.google-analytics.com/ga.js"></script>
            <script type="text/javascript">
                %(create_tracker_command)s
                %(commands)s
            </script>
            """,
        "universal-async":
            """
            <script type="text/javascript">
              (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
              (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
              m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
              })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
              %(create_tracker_command)s
              %(commands)s
            </script>
            """,
        "universal-sync":
            """
            <script type="text/javascript">
              (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
              (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
              m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
              })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
              %(create_tracker_command)s;
              ga(function () {
                  %(commands)s
              });
            </script>
            """,
    }

    declaring_tracker = Event()

    def get_analytics_script(self,
        publishable = None,
        commands = None,
        async = True
    ):
        config = Configuration.instance
        api = config.google_analytics_api
        classic_api = (api == "ga.js")

        if publishable is None:
            publishable = context.get("publishable")

        event = self.declaring_tracker(
            api = api,
            publishable = publishable,
            account = config.get_setting("google_analytics_account"),
            tracker_parameters = None if classic_api else {},
            domain = config.get_setting("google_analytics_domain"),
            template =
                self.inclusion_code[api + ("-async" if async else "-sync")],
            values =
                None if classic_api else self.get_analytics_values(publishable),
            commands = commands or []
        )

        if not event.account:
            return ""

        commands = event.commands
        parameters = {}

        if classic_api:
            create_commands = [("_setAccount", event.account)]

            if event.domain:
                create_commands.append(("_setDomainName", event.domain))

            parameters["create_tracker_command"] = \
                self._serialize_ga_commands(create_commands)

            parameters["commands"] = self._serialize_ga_commands(commands)
        else:
            if event.domain:
                event.tracker_parameters["cookieDomain"] = event.domain

            parameters["create_tracker_command"] = \
                self._serialize_universal_commands([(
                    "create",
                    event.account,
                    event.tracker_parameters
                )])

            if event.values:
                commands.insert(0, ("set", event.values))

            parameters["commands"] = \
                self._serialize_universal_commands(commands)

        return event.template % parameters

    def get_analytics_page_hit_script(self, publishable = None):

        api = Configuration.instance.google_analytics_api

        if api == "ga.js":
            commands = [("_trackPageview",)]
        elif api == "universal":
            commands = [("send", "pageview")]
        else:
            return ""

        return self.get_analytics_script(
            publishable = publishable,
            commands = commands
        )

    def get_analytics_values(self, publishable):
        return {
            # Custom dimension: language
            "dimension1": get_language() or "",

            # Custom dimension: roles
            "dimension2":
                self.serialize_collection(
                    role.id for role in get_current_user().iter_roles()
                ),

            # Custom dimension: path
            "dimension3":
                "" if publishable is None
                else self.serialize_collection(
                    ancestor.id
                    for ancestor
                    in reversed(list(publishable.ascend_tree(True)))
                )
        }

    def serialize_collection(self, value):
        return "-" + "-".join(str(item) for item in value) + "-"

    def _serialize_ga_commands(self, commands):
        return "\n".join(
            "_gaq.push([%s]);" % (", ".join(dumps(arg) for arg in cmd))
            for cmd in commands
        )

    def _serialize_universal_commands(self, commands):
        return "\n".join(
            "ga(%s);" % (", ".join(dumps(arg) for arg in cmd))
            for cmd in commands
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define("Configuration.google_analytics_api",
    ca = u"API de Google Analytics",
    es = u"API de Google Analytics",
    en = u"Google Analytics API"
)

for cls_name in ("Configuration", "Website"):
    translations.define(cls_name + ".services.google_analytics",
        ca = u"Google Analytics",
        es = u"Google Analytics",
        en = u"Google Analytics"
    )

    translations.define(cls_name + ".google_analytics_account",
        ca = u"Compte de Google Analytics",
        es = u"Cuenta de Google Analytics",
        en = u"Google Analytics account"
    )

    translations.define(cls_name + ".google_analytics_domain",
        ca = u"Domini per Google Analytics",
        es = u"Dominio para Google Analytics",
        en = u"Domain for Google Analytics"
    )

translations.define("Document.ga_tracking_enabled",
    ca = u"Habilitar seguiment amb Google Analytics",
    es = u"Habilitar seguimiento con Google Analytics",
    en = u"Track with Google Analytics"
)

# Client redirections
#------------------------------------------------------------------------------
translations.define("woost.extensions.googleanalytics.redirection_title",
    ca = u"Obrint recurs",
    es = u"Abriendo recurso",
    en = u"Downloading resource"
)

translations.define("woost.extensions.googleanalytics.redirection_body",
    ca = lambda url:
        u"S'està obrint el recurs sol·licitat. Si no s'obre automàticament, "
        u"si us plau faci clic <a href='%s'>aquí</a>."
        % url,
    es = lambda url:
        u"Se está abriendo el recurso solicitado. Si no se abre "
        u"automáticamente, por favor haga clic <a href='%s'>aquí</a>."
        % url,
    en = lambda url:
        u"The page or file you requested is being downloaded. If the download "
        u"doesn't start automatically, please click <a href='%s'>here</a>."
        % url
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail import schema
from cocktail.controllers import (
    Controller,
    request_property,
    FormProcessor
)
from woost.models import Publishable, get_current_website
from woost.extensions.ecommerce.basket import Basket
from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder
from woost.extensions.ecommerce.orderstepcontroller import ProceedForm
from woost.extensions.payments import PaymentsExtension


class SummaryController(FormProcessor, Controller):

    is_transactional = True

    class SubmitOrderForm(ProceedForm):

        order = None

        @request_property
        def errors(self):
            return schema.ErrorList(ECommerceOrder.get_errors(Basket.get()))

        def submit(self):
            ProceedForm.submit(self)
            self.order = Basket.pop()
            self.order.status = "payment_pending"
            self.order.update_cost()

        def after_submit(self):
            payments = PaymentsExtension.instance
            website = get_current_website()
            payment_gateway = website.ecommerce_payment_gateway

            # Redirect the user to the payment gateway
            if payments.enabled \
            and payment_gateway \
            and self.order.payment_type == "payment_gateway":
                payment_gateway.initiate_payment(self.order.id)

            # No payment gateway available, redirect the user to the success
            # page; the payment will have to be handled manually by the site's
            # personnel
            else:
                raise cherrypy.HTTPRedirect(
                    Publishable.require_instance(
                        qname = "woost.extensions.ecommerce.success_page"
                    ).get_uri(
                        parameters = {"order": self.order.id}
                    )
                )

    @request_property
    def checkout_schema(self):
        return Basket.get().get_public_schema()

    @request_property
    def output(self):
        output = Controller.output(self)
        output["checkout_schema"] = self.checkout_schema
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.controllers import request_property
from woost.controllers.backoffice.useractions import get_user_action
from woost.controllers.backoffice.editcontroller import EditController


class ReferencesController(EditController):

    view_class = "woost.views.BackOfficeReferencesView"

    @request_property
    def references(self):
        return get_user_action("references").references

    @request_property
    def output(self):
        output = EditController.output(self)
        output.update(
            selected_action = get_user_action("references"),
            references = self.references
        )
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from woost.models import Extension, Configuration, Website


translations.define("ExternalFilesExtension",
    ca = u"Fitxers externs",
    es = u"Ficheros externos",
    en = u"External files"
)

translations.define("ExternalFilesExtension-plural",
    ca = u"Fitxers externs",
    es = u"Ficheros externos",
    en = u"External files"
)


class ExternalFilesExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afegeix suport per fitxers externs""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para ficheros externos""",
            "es"
        )
        self.set("description",
            u"""Adds support for external files""",
            "en"
        )

    def _load(self):
        from woost.extensions.externalfiles import (
            strings,
            configuration,
            website,
            item,
            publishable
        )

        self.install()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import OrderedSet, InstrumentedOrderedSet
from cocktail import schema
from cocktail.translations import translations
from cocktail.persistence import transactional
from cocktail.controllers import request_property, get_parameter
from woost.models import (
    Item,
    ModifyPermission,
    ModifyMemberPermission,
    changeset_context,
    get_current_user
)
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class DragAndDropController(BaseBackOfficeController):

    rendering_format = "json"

    @request_property
    def dragged_object(self):
        return get_parameter(
            schema.Reference("dragged_object", type = Item)
        )

    @request_property
    def target_object(self):
        return get_parameter(
            schema.Reference("target_object", type = Item)
        )

    @request_property
    def target_member(self):
        eligible_members = [member.name  for member in self.eligible_members]

        if eligible_members:
            member_name = get_parameter(
                schema.String("target_member", enumeration = eligible_members)
            )
            if member_name:
                return self.target_object.__class__.get_member(member_name)

    @request_property
    def eligible_members(self):

        dragged_object = self.dragged_object
        target_object = self.target_object

        if not dragged_object or not target_object:
            return []

        user = get_current_user()

        return [
            member
            for member in target_object.__class__.iter_members()
            if member.visible
            and isinstance(member, schema.RelationMember)
            and member.is_persistent_relation
            and isinstance(dragged_object, member.related_type)
            and user.has_permission(ModifyMemberPermission, member = member)
        ]

    @request_property
    def sibling(self):
        target_member = self.target_member
        if target_member and isinstance(target_member, schema.Collection):
            return get_parameter(
                schema.Reference("sibling",
                    type = target_member.related_type,
                    enumeration = self.target_object.get(target_member)
                )
            )

    @request_property
    def relative_placement(self):
        return get_parameter(
            schema.String("relative_placement", enumeration = ["after", "before"])
        )

    @request_property
    def valid(self):
        return (
            self.dragged_object
            and self.target_object
            and self.target_member
            and (not self.sibling or self.relative_placement)
        )

    def submit(self):

        # TODO: cycle prevention
        # TODO: support for inserting before a sibling
        # TODO: dropping objects on objects (inferring/asking the target member)

        @transactional()
        def relate():
            dragged_object = self.dragged_object
            target_object = self.target_object
            target_member = self.target_member

            user = get_current_user()
            user.require_permission(ModifyPermission, target = dragged_object)
            user.require_permission(ModifyPermission, target = target_object)

            with changeset_context(user):
                if isinstance(target_member, schema.Reference):
                    target_object.set(target_member, dragged_object)
                else:
                    collection = target_object.get(target_member)

                    sibling = self.sibling
                    relative_placement = self.relative_placement

                    if sibling:
                        try:
                            current_index = collection.index(dragged_object)
                        except ValueError:
                            current_index = None
                        index = collection.index(sibling)

                        if relative_placement == "before":
                            if current_index is not None and current_index < index:
                                index -= 1
                        elif relative_placement == "after":
                            if current_index is None or current_index >= index:
                                index += 1

                        if isinstance(
                            collection,
                            (OrderedSet, InstrumentedOrderedSet)
                        ):
                            kwargs = {"relocate": True}
                        else:
                            kwargs = {}

                        collection.insert(index, dragged_object, **kwargs)
                    else:
                        collection.append(dragged_object)
        relate()

        notify_user(
            translations(
                "woost.controllers.backoffice.DragAndDropController"
                ".drop_notification",
                dragged_object = self.dragged_object,
                target_object = self.target_object,
                target_member = self.target_member
            ),
            "success",
            transient = False
        )

    @request_property
    def output(self):
        return {}




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.modeling import ListWrapper
from cocktail.translations import translations, get_language
from cocktail import schema
from woost.models import Extension, Configuration

translations.define("GoogleSearchExtension",
    ca = u"Cercador de Google",
    es = u"Buscador de Google",
    en = u"Google search"
)

translations.define("GoogleSearchExtension-plural",
    ca = u"Cercadors de Google",
    es = u"Buscadores de Google",
    en = u"Google search"
)


class GoogleSearchExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona els elements essencials per implementar el cercador
            de Google""",
            "ca"
        )
        self.set("description",
            u"""Proporciona los elementos esenciales para implementar el
            buscador de Google""",
            "es"
        )
        self.set("description",
            u"""Provides the essential elements to set up Google Search""",
            "en"
        )

    def _load(self):
        from woost.extensions.googlesearch import (
            strings,
            configuration,
            website
        )

    def search(self,
        query,
        page = 0,
        page_size = 10,
        language = None,
        filter = True,
        results_per_page = None):
        """Returns Google CSE results for the given query.

        @param query: The query to search.
        @type query: unicode

        @param page: The ordinal index of the results page to return, starting
            at 0.
        @type page: int

        @param page_size: The maximum number of results per page.
        @type page_size: int

        @param language: Restricts search results to matches for the given
            language. The language must be indicated using a two letter ISO
            language code.
        @type language: str

        @param filter: Indicates if Google should apply filtering over the
            search results (ie. to remove redundant matches).
        @type filter: bool

        @return: The results for the given query.
        @rtype: L{GoogleSearchResultsList}
        """
        from woost.extensions.googlesearch.search import google_cse_search
        return google_cse_search(
            Configuration.instance.get_setting("google_search_engine_id"),
            query,
            page = page,
            page_size = page_size,
            language = language,
            filter = filter,
            results_per_page = results_per_page
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema

# Add an extension property to control the default member visibility on item listings
schema.Member.listed_by_default = True
schema.Collection.listed_by_default = False
schema.CodeBlock.listed_by_default = False

# Add an extension property to show/hide the 'Element' column on listings
schema.Schema.show_element_in_listings = True

# Add an extension property to show/hide the 'Type' column on listings
schema.Schema.show_type_in_listings = True

# Add an extension property to indicate that selecting a model on the
# backoffice main menu should skip its listing and jump to editing its
# singleton instance instead.
schema.Schema.skip_listing = False

# Add an extension property to indicate if members should be visible by users
schema.Member.visible = True

# Add an extension property to indicate if schemas should be instantiable by
# users
schema.Schema.instantiable = True

# Add an extension property to indicate if members should be editable by users
schema.Member.editable = True

# Add an extesnion property to indiciate if members should be shown in detailed view
schema.Member.visible_in_detail_view = True

# Add an extension property to group types
schema.Schema.type_group = None

# Add an extension property to indicate if relations should be excluded if no
# relatable elements exist
schema.Collection.exclude_when_empty = False

# Add an extension property to determine if members should participate in item
# revisions
schema.Member.versioned = True

# Extension property that makes it easier to customize the edit view for a
# collection in the backoffice
schema.Collection.edit_view = None

# Extension property that sets the default type that should be shown by default
# when opening an item selector for the indicated property
schema.RelationMember.selector_default_type = None

# Extension property that allows to indicate that specific members don't modify
# the 'last_update_time' member of items when changed
schema.Member.affects_last_update_time = True

# Extension property that allows hiding relations in the ReferenceList view
schema.RelationMember.visible_in_reference_list = True

# Extension property to select which members should be synchronizable across
# separate site installations
schema.Member.synchronizable = True




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from cocktail.styled import styled
from cocktail.events import when
from cocktail.persistence import datastore
from woost import app
from woost.models.publishable import Publishable
from woost.models.file import File
from woost.models.user import User

debug = True
enabled = True
filesystem_encoding = "utf-8"

members_affecting_static_publication = set([
    File.title,
    File.file_name,
    Publishable.enabled,
    Publishable.per_language_publication,
    Publishable.enabled_translations,
    Publishable.start_date,
    Publishable.end_date,
    Publishable.access_level
])

def encode_filename(link, encoding = None):
    if encoding is None:
        encoding = filesystem_encoding
    return link.encode(encoding) if isinstance(link, unicode) else link

def create_links(file, links = None, encoding = None):

    if not file.is_inserted:
        return

    anonymous = User.require_instance(qname = "woost.anonymous_user")

    if not file.is_accessible(anonymous):
        return

    if links is None:
        links = get_links(file)

    linked_file = file.file_path
    linked_file = encode_filename(linked_file, encoding)

    for link in links:

        link = encode_filename(link, encoding)

        if debug:
            print styled("STATIC PUBLICATION", "white", "green"),
            print "Adding link for #%s (%s -> %s)" % (
                styled(file.id, style = "bold"),
                styled(link, "yellow"),
                styled(linked_file, "brown")
            )

        # Delete the link if it already existed
        try:
            os.remove(link)
        except OSError:
            pass

        # Create the new link
        os.symlink(linked_file, link)

def remove_links(file, links = None, encoding = None):

    if links is None:
        links = get_links(file)

    for link in links:
        link = encode_filename(link, encoding)
        if os.path.lexists(link):
            if debug:
                print styled("STATIC PUBLICATION", "white", "red"),
                print "Removing link for #%s (%s)" % (
                    styled(file.id, style = "bold"),
                    styled(link, "yellow")
                )
            os.remove(link)

def get_links(file):
    return [
        app.path("static", app.url_resolver.get_path(file, language))
        for language in (file.translations or [None])
    ]

if hasattr(os, "symlink"):

    @when(File.inserted)
    @when(File.changed)
    @when(File.deleting)
    def _schedule_links_update(e):

        if not enabled or not app.root:
            return

        file = e.source
        member = getattr(e, "member", None)

        if member is not None:

            if not file.is_inserted:
                return

            if member not in members_affecting_static_publication:
                return

        # Store the current links, to be removed when the current transaction is
        # committed
        key = "woost.models.File.old_links-%d" % file.id
        old_links = datastore.get_transaction_value(key)

        if old_links is None:
            old_links = set()
            datastore.set_transaction_value(key, old_links)

        old_links.update(get_links(file))

        # Add an after commit hook to remove/create the file's links
        datastore.unique_after_commit_hook(
            "woost.models.File.update_static_links-%d" % file.id,
            _update_links_after_commit,
            file,
            old_links
        )

    def _update_links_after_commit(commit_successful, file, old_links):
        if commit_successful:
            remove_links(file, old_links)
            create_links(file)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.translations.helpers import plural2

translations.define("woost.actions.survey_results",
    ca = u"Veure resultats",
    es = u"Ver resultados",
    en = u"View results"
)

# Survey
#------------------------------------------------------------------------------
translations.define("Survey",
    ca = u"Enquesta",
    es = u"Encuesta",
    en = u"Survey"
)

translations.define("Survey-plural",
    ca = u"Enquesta",
    es = u"Encuesta",
    en = u"Survey"
)

translations.define("Survey.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("Survey.questions",
    ca = u"Preguntes",
    es = u"Preguntas",
    en = u"Questions"
)

# SurveyQuestion
#------------------------------------------------------------------------------
translations.define("SurveyQuestion",
    ca = u"Pregunta d'enquesta",
    es = u"Pregunta de encuesta",
    en = u"Survey question"
)

translations.define("SurveyQuestion-plural",
    ca = u"Preguntes d'enquestes",
    es = u"Preguntas de encuesta",
    en = u"Survey questions"
)

translations.define("SurveyQuestion.title",
    ca = u"Enunciat",
    es = u"Enunciado",
    en = u"Question"
)

translations.define("SurveyQuestion.survey",
    ca = u"Enquesta",
    es = u"Encuesta",
    en = u"Survey"
)

translations.define("SurveyQuestion.active",
    ca = u"Activa",
    es = u"Activa",
    en = u"Active"
)

# SurveyOptionsQuestion
#------------------------------------------------------------------------------
translations.define("SurveyOptionsQuestion",
    ca = u"Pregunta amb opcions",
    es = u"Pregunta con opciones",
    en = u"Options question"
)

translations.define("SurveyOptionsQuestion-plural",
    ca = u"Preguntes amb opcions",
    es = u"Preguntas con opciones",
    en = u"Options questions"
)

translations.define("SurveyOptionsQuestion.options",
    ca = u"Opcions",
    es = u"Opciones",
    en = u"Options"
)

# SurveyOption
#------------------------------------------------------------------------------
translations.define("SurveyOption",
    ca = u"Opció",
    es = u"Opción",
    en = u"Option"
)

translations.define("SurveyOption-plural",
    ca = u"Opcions",
    es = u"Opciones",
    en = u"Options"
)

translations.define("SurveyOption.title",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("SurveyOption.active",
    ca = u"Activa",
    es = u"Activa",
    en = u"Active"
)

# SurveyTextQuestion
#------------------------------------------------------------------------------
translations.define("SurveyTextQuestion",
    ca = u"Pregunta de text lliure",
    es = u"Pregunta de texto libre",
    en = u"Text question"
)

translations.define("SurveyTextQuestion-plural",
    ca = u"Preguntes de text lliure",
    es = u"Preguntas de texto libre",
    en = u"Text questions"
)

translations.define("SurveyTextQuestion.mandatory",
    ca = u"Obligatòria",
    es = u"Obligatoria",
    en = u"Mandatory"
)

# SurveyBlock
#------------------------------------------------------------------------------
translations.define("SurveyBlock",
    ca = u"Formulari d'enquesta",
    es = u"Formulario de encuesta",
    en = u"Survey form"
)

translations.define("SurveyBlock-plural",
    ca = u"Formularis d'enquesta",
    es = u"Formularios de encuesta",
    en = u"Survey forms"
)

translations.define("SurveyBlock.survey",
    ca = u"Enquesta",
    es = u"Encuesta",
    en = u"Survey"
)

translations.define("SurveyBlock.redirection",
    ca = u"Redirecció",
    es = u"Redirección",
    en = u"Redirection"
)

translations.define("SurveyBlock.redirection-explanation",
    ca = u"La pàgina on es rediregeix als usuaris que completin l'enquesta.",
    es = u"La página donde se redirige a los usuarios que completen la "
         "encuesta.",
    en = u"The page where users who complete the survey will be redirected."
)

# SurveyResultsView
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.surveys.SurveyResultsView.answers_heading",
    ca = u"Respostes",
    es = u"Respuestas",
    en = u"Answers"
)

translations.define(
    "woost.extensions.surveys.SurveyResultsView.submission_count",
    ca = lambda count:
        u"L'enquesta " + (
            u"encara no ha estat emplenada per cap usuari"
            if not count
            else (
                u"ha estat emplenada per <strong>%d</strong> " % count
                + plural2(count, u"usuari", u"usuaris")
            )
            + "."
        ),
    es = lambda count:
        u"La encuesta " + (
            u"todavía no sido contestada por ningún usuario"
            if not count
            else (
                u"ha sido contestada por <strong>%d</strong> " % count
                + plural2(count, u"usuario", u"usuarios")
            )
            + "."
        ),
    en = lambda count:
        u"The survey " + (
            u"hasn't been answered by anybody yet"
            if not count
            else (
                u"has been answered by <strong>%d</strong> " % count
                + plural2(count, u"user", u"users")
            )
            + "."
        )
)

translations.define(
    "woost.extensions.surveys.SurveyResultsView.download_excel_link",
    ca = u"Descarregar totes les respostes",
    es = u"Descargar todas las respuestas",
    en = u"Download all answers"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models.permission import ContentPermission, ReadPermission
from woost.models.rendering.imagefactory import ImageFactory
from woost.models.messagestyles import permission_doesnt_match_style


class RenderPermission(ContentPermission):
    """Permission to obtain images representing instances of a content type."""

    instantiable = True

    factories = schema.Collection(
        items = schema.Reference(type = ImageFactory),
        related_end = schema.Collection(),
        edit_control = "cocktail.html.CheckList",
        searchable = False
    )

    def match(self, target, image_factory, verbose = False):

        if self.factories and image_factory not in self.factories:
            print permission_doesnt_match_style("factory doesn't match")
            return False

        return ContentPermission.match(self, target, verbose)

    @classmethod
    def permission_not_found(cls, user, verbose = False, **context):
        # If no specific render permission is found, a read permission will do
        return user.has_permission(
            ReadPermission,
            target = context["target"],
            verbose = verbose
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.block import Block
from woost.extensions.campaign3.campaignmonitorlist import \
    CampaignMonitorList


class SubscriptionFormBlock(Block):

    instantiable = True
    type_group = "blocks.forms"

    members_order = ["subscriber_model", "lists"]

    default_controller = \
        "woost.extensions.campaign3.subscriptioncontroller.SubscriptionController"

    subscriber_model = schema.String(
        required = True,
        default = "woost.extensions.campaign3.subscriber.Subscriber",
        text_search = False,
        member_group = "content"
    )

    lists = schema.Collection(
        items = schema.Reference(
            type = CampaignMonitorList
        ),
        min = 1,
        member_group = "content",
        listed_by_default = False
    )

    view_class = schema.String(
        required = True,
        shadows_attribute = True,
        before_member = "controller",
        member_group = "behavior"
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension, Controller, Configuration
from woost.models.rendering import ChainRenderer


translations.define("CampaignMonitor3Extension",
    ca = u"Campaign Monitor v3",
    es = u"Campaign Monitor v3",
    en = u"Campaign Monitor v3"
)

translations.define("CampaignMonitor3Extension-plural",
    ca = u"Campaign Monitor v3",
    es = u"Campaign Monitor v3",
    en = u"Campaign Monitor v3"
)


class CampaignMonitor3Extension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet la integració amb el sistema de mailing Campaign Monitor""",
            "ca"
        )
        self.set("description",
            u"""Permite la integración con el sistema de mailing Campaign Monitor""",
            "es"
        )
        self.set("description",
            u"""Allows the integration with the Campaign Monitor mailing system""",
            "en"
        )

    def _load(self):

        from woost.extensions.campaign3 import (
            strings,
            configuration,
            website,
            campaignmonitorlist,
            subscriptionformblock
        )

    def _install(self):
        pass




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			Jun 2009
"""
import os
from unittest import TestCase
from cocktail.tests.persistence.tempstoragemixin import TempStorageMixin

class BaseTestCase(TempStorageMixin, TestCase):

    def setUp(self):

        from woost import app
        from woost.models import Configuration, User, Role
        from woost.models.trigger import get_triggers_enabled, set_triggers_enabled
        from woost.controllers.installer import Installer

        self.__prev_installation_id = app.installation_id
        self.__prev_triggers_enabled = get_triggers_enabled()
        app.installation_id = "TEST"
        set_triggers_enabled(False)

        TempStorageMixin.setUp(self)

        app.root = os.path.join(self._temp_dir, "test_project")
        installer = Installer()
        installer.create_project({"project_path": app.root})

        # Configuration
        self.config = Configuration(qname = "woost.configuration")
        self.config.insert()

        # Roles and users
        self.anonymous_role = Role(qname = "woost.anonymous")
        self.anonymous_role.insert()

        self.anonymous_user = User(qname = "woost.anonymous_user")
        self.anonymous_user.roles.append(self.anonymous_role)
        self.anonymous_user.insert()

        self.everybody_role = Role(qname = "woost.everybody")
        self.everybody_role.insert()

        self.authenticated_role = Role(qname = "woost.authenticated")
        self.authenticated_role.insert()

        set_triggers_enabled(True)

    def tearDown(self):
        from woost import app
        from woost.models import staticpublication
        from woost.models.trigger import set_triggers_enabled

        app.installation_id = self.__prev_installation_id
        set_triggers_enabled(self.__prev_triggers_enabled)

        TempStorageMixin.tearDown(self)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from woost.models import Extension, Configuration


translations.define("AudioExtension",
    ca = u"Àudio",
    es = u"Audio",
    en = u"Audio"
)

translations.define("AudioExtension-plural",
    ca = u"Àudio",
    es = u"Audio",
    en = u"Audio"
)


class AudioExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Reproducció de fitxers d'àudio amb recodificació automàtica a
            múltiples formats.""",
            "ca"
        )
        self.set("description",
            u"""Reproducción de ficheros de audio con recodificación automática
            a múltiples formatos.""",
            "es"
        )
        self.set("description",
            u"""Audio file player with transparent encoding in multiple
            formats.""",
            "en"
        )

    def _load(self):

        from woost.extensions.audio import (
            strings,
            configuration,
            audiodecoder,
            audioencoder
        )

        # Expose the controller for serving audio files in multiple encodings
        from woost.controllers.cmscontroller import CMSController
        from woost.extensions.audio.audioencodingcontroller \
            import AudioEncodingController

        CMSController.audio = AudioEncodingController

        self.install()
        self.register_view_factory()

    def _install(self):
        self.create_default_decoders()
        self.create_default_encoders()

    def create_default_decoders(self):

        from woost.extensions.audio.audiodecoder import AudioDecoder
        config = Configuration.instance

        mp3 = AudioDecoder()
        mp3.mime_type = "audio/mpeg"
        mp3.command = '/usr/bin/mpg321 "%s" -w -'
        mp3.insert()
        config.audio_decoders.append(mp3)

        ogg = AudioDecoder()
        ogg.mime_type = "audio/ogg"
        ogg.command = '/usr/bin/oggdec -Q -o - "%s"'
        ogg.insert()
        config.audio_decoders.append(ogg)

        flac = AudioDecoder()
        flac.mime_type = "audio/flac"
        flac.command = '/usr/bin/flac -dsc "%s"'
        flac.insert()
        config.audio_decoders.append(flac)

    def create_default_encoders(self):

        from woost.extensions.audio.audioencoder import AudioEncoder
        config = Configuration.instance

        mp3 = AudioEncoder()
        mp3.identifier = "mp3-128"
        mp3.mime_type = "audio/mpeg"
        mp3.extension = ".mp3"
        mp3.command = "/usr/bin/lame --quiet -b 128 - %s"
        mp3.insert()
        config.audio_encoders.append(mp3)

        ogg = AudioEncoder()
        ogg.identifier = "ogg-q5"
        ogg.mime_type = "audio/ogg"
        ogg.extension = ".ogg"
        ogg.command = "/usr/bin/oggenc -q 5 - -o %s"
        ogg.insert()
        config.audio_encoders.append(ogg)

    def register_view_factory(self):

        from woost.models import Publishable
        from woost.extensions.audio.audioplayer import AudioPlayer
        from woost.views.viewfactory import publishable_view_factory

        def audio_player(item, parameters):
            if item.resource_type == "audio":
                player = AudioPlayer()
                player.file = item
                return player

        publishable_view_factory.register_first(Publishable, "audio_player", audio_player)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Website
#------------------------------------------------------------------------------
translations.define("Website.notices",
    ca = u"Avisos",
    es = u"Avisos",
    en = u"Notices"
)

# NoticesBox
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.notices.NoticesBox.close_notice_button",
    ca = u"Ocultar aquest missatge",
    es = u"Ocultar este mensaje",
    en = u"Hide this message"
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from datetime import datetime
from cocktail.modeling import getter, classgetter
from cocktail.events import event_handler
from cocktail.pkgutils import import_object
from cocktail.translations import (
    translations,
    get_language,
    require_language
)
from cocktail import schema
from cocktail.schema.expressions import Expression
from cocktail.persistence import datastore, MultipleValuesIndex
from cocktail.controllers import (
    make_uri,
    percent_encode_uri,
    Location
)
from cocktail.html.datadisplay import display_factory
from woost import app
from .enabledtranslations import auto_enables_translations
from .localemember import LocaleMember
from .item import Item
from .role import Role
from .usersession import get_current_user
from .websitesession import get_current_website
from .permission import (
    ReadPermission,
    ReadTranslationPermission,
    PermissionExpression
)
from .caching import CachingPolicy

WEBSITE_PUB_INDEX_KEY = "woost.models.Publishable.per_website_publication_index"


@auto_enables_translations
class Publishable(Item):
    """Base class for all site elements suitable for publication."""

    instantiable = False
    cacheable = True
    edit_view = "woost.views.PublishableFieldsView"
    edit_node_class = (
        "woost.controllers.backoffice.enabledtranslationseditnode."
        "EnabledTranslationsEditNode"
    )
    backoffice_heading_view = "woost.views.BackOfficePublishableHeading"

    type_group = "publishable"

    groups_order = [
        "navigation",
        "presentation",
        "presentation.behavior",
        "presentation.format",
        "publication"
    ]

    members_order = [
        "controller",
        "mime_type",
        "resource_type",
        "encoding",
        "parent",
        "path",
        "full_path",
        "hidden",
        "login_page",
        "per_language_publication",
        "enabled",
        "enabled_translations",
        "websites",
        "access_level",
        "start_date",
        "end_date",
        "requires_https",
        "caching_policies"
    ]

    mime_type = schema.String(
        required = True,
        default = "text/html",
        text_search = False,
        format = r"^[^/]+/[^/]+$",
        listed_by_default = False,
        member_group = "presentation.format"
    )

    resource_type = schema.String(
        indexed = True,
        text_search = False,
        editable = False,
        enumeration = (
            "document",
            "image",
            "audio",
            "video",
            "package",
            "html_resource",
            "other"
        ),
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "woost.models.Publishable.resource_type " + value,
                 language,
                **kwargs
            ),
        listed_by_default = False,
        member_group = "presentation.format"
    )

    encoding = schema.String(
        listed_by_default = False,
        text_search = False,
        default = "utf-8",
        member_group = "presentation.format"
    )

    controller = schema.Reference(
        type = "woost.models.Controller",
        indexed = True,
        bidirectional = True,
        listed_by_default = False,
        member_group = "presentation.behavior"
    )

    def resolve_controller(self):
        if self.controller and self.controller.python_name:
            return import_object(self.controller.python_name)

    parent = schema.Reference(
        type = "woost.models.Document",
        bidirectional = True,
        related_key = "children",
        listed_by_default = False,
        member_group = "navigation"
    )

    path = schema.String(
        max = 1024,
        indexed = True,
        listed_by_default = False,
        text_search = False,
        member_group = "navigation"
    )

    full_path = schema.String(
        indexed = True,
        unique = True,
        editable = False,
        text_search = False,
        listed_by_default = False,
        member_group = "navigation"
    )

    hidden = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "navigation"
    )

    login_page = schema.Reference(
        listed_by_default = False,
        member_group = "navigation"
    )

    per_language_publication = schema.Boolean(
        required = True,
        default = False,
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    enabled = schema.Boolean(
        required = True,
        default = True,
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    enabled_translations = schema.Collection(
        items = LocaleMember(),
        default_type = set,
        indexed = True,
        edit_control = "woost.views.EnabledTranslationsSelector",
        listed_by_default = False,
        member_group = "publication"
    )

    websites = schema.Collection(
        items = "woost.models.Website",
        bidirectional = True,
        related_key = "specific_content",
        edit_control = "cocktail.html.CheckList",
        member_group = "publication"
    )

    access_level = schema.Reference(
        type = "woost.models.AccessLevel",
        bidirectional = True,
        indexed = True,
        edit_control = display_factory(
            "cocktail.html.RadioSelector",
            empty_option_displayed = True
        ),
        listed_by_default = False,
        member_group = "publication"
    )

    start_date = schema.DateTime(
        indexed = True,
        listed_by_default = False,
        affects_cache_expiration = True,
        member_group = "publication"
    )

    end_date = schema.DateTime(
        indexed = True,
        min = start_date,
        listed_by_default = False,
        affects_cache_expiration = True,
        member_group = "publication"
    )

    requires_https = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "publication"
    )

    caching_policies = schema.Collection(
        items = schema.Reference(type = CachingPolicy),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "publication"
    )

    def get_effective_caching_policy(self, **context):

        from woost.models import Configuration

        policies = [
            ((-policy.important, 1), policy)
            for policy in self.caching_policies
        ]
        policies.extend(
            ((-policy.important, 2), policy)
            for policy in Configuration.instance.caching_policies
        )
        policies.sort()

        for criteria, policy in policies:
            if policy.applies_to(self, **context):
                return policy

    @event_handler
    def handle_changed(cls, event):

        member = event.member
        publishable = event.source

        if member.name == "path":
            publishable._update_path(publishable.parent, event.value)

        elif member.name == "parent":
            publishable._update_path(event.value, publishable.path)

            # If the parent element is specific to one or more websites, its
            # descendants will automatically share that specificity
            if event.value:
                publishable.websites = list(event.value.websites)
            else:
                publishable.websites = []

        elif member.name == "mime_type":
            if event.value is None:
                publishable.resource_type = None
            else:
                publishable.resource_type = \
                    get_category_from_mime_type(event.value)

    @event_handler
    def handle_related(cls, event):
        if event.member is cls.websites:
            publishable = event.source
            website = event.related_object

            # Update the index
            if publishable.is_inserted and website.is_inserted:
                index = cls.per_website_publication_index
                index.remove(None, publishable.id)
                index.add(website.id, publishable.id)

    @event_handler
    def handle_unrelated(cls, event):
        if event.member is cls.websites:
            publishable = event.source
            website = event.related_object

            index = cls.per_website_publication_index
            index.remove(website.id, publishable.id)

            # Now available to any website
            if publishable.is_inserted and not publishable.websites:
                index.add(None, publishable.id)

    @event_handler
    def handle_inserted(cls, event):
        event.source.__insert_into_per_website_publication_index()

    @event_handler
    def handle_deleted(cls, event):
        cls.per_website_publication_index.remove(None, event.source.id)

    def __insert_into_per_website_publication_index(self):

        index = self.__class__.per_website_publication_index

        # Available to any website
        if not self.websites:
            index.add(None, self.id)

        # Restricted to a subset of websites
        else:
            for website in self.websites:
                if website.is_inserted:
                    index.add(website.id, self.id)

    @classgetter
    def per_website_publication_index(cls):
        """A database index that enumerates content exclusive to one or more
        websites.
        """
        index = datastore.root.get(WEBSITE_PUB_INDEX_KEY)

        if index is None:
            index = datastore.root.get(WEBSITE_PUB_INDEX_KEY)
            if index is None:
                index = MultipleValuesIndex()
                datastore.root[WEBSITE_PUB_INDEX_KEY] = index

        return index

    @event_handler
    def handle_rebuilding_indexes(cls, e):
        cls.rebuild_per_website_publication_index(verbose = e.verbose)

    @classmethod
    def rebuild_per_website_publication_index(cls, verbose = False):
        if verbose:
            print "Rebuilding the Publishable/Website index"
        del datastore.root[WEBSITE_PUB_INDEX_KEY]
        for publishable in cls.select():
            publishable.__insert_into_per_website_publication_index()

    @classgetter
    def per_language_publication_index(cls):
        return datastore.root[WEBSITE_PUB_INDEX_KEY]

    def _update_path(self, parent, path):

        parent_path = parent and parent.full_path

        if parent_path and path:
            self.full_path = parent_path + "/" + path
        else:
            self.full_path = path

    def get_ancestor(self, depth):
        """Obtain one of the item's ancestors, given its depth in the document
        tree.

        @param depth: The depth level of the ancestor to obtain, with 0
            indicating the root of the tree. Negative indices are accepted, and
            they reverse the traversal order (-1 will point to the item itself,
            -2 to its parent, and so on).
        @type depth: int

        @return: The requested ancestor, or None if there is no ancestor with
            the indicated depth.
        @rtype: L{Publishable}
        """
        tree_line = list(self.ascend_tree(include_self = True))
        tree_line.reverse()
        try:
            return tree_line[depth]
        except IndexError:
            return None

    def ascend_tree(self, include_self = False):
        """Iterate over the item's ancestors, moving towards the root of the
        document tree.

        @param include_self: Indicates if the object itself should be included
            in the iteration.
        @type include_self: bool

        @return: An iterable sequence of pages.
        @rtype: L{Document} iterable sequence
        """
        publishable = self if include_self else self.parent
        while publishable is not None:
            yield publishable
            publishable = publishable.parent

    def descend_tree(self, include_self = False):
        """Iterate over the item's descendants.

        @param include_self: Indicates if the object itself should be included
            in the iteration.
        @type include_self: bool

        @return: An iterable sequence of publishable elements.
        @rtype: L{Publishable} iterable sequence
        """
        if include_self:
            yield self

    def descends_from(self, page):
        """Indicates if the object descends from the given document.

        @param page: The hypothetical ancestor of the page.
        @type page: L{Document<woost.models.document.Document>}

        @return: True if the object is contained inside the given document or
            one of its descendants, or if it *is* the given document. False in
            any other case.
        @rtype: bool
        """
        ancestor = self

        while ancestor is not None:
            if ancestor is page:
                return True
            ancestor = ancestor.parent

        return False

    def is_home_page(self):
        """Indicates if the object is the home page for any website.
        @rtype: bool
        """
        from woost.models import Website
        return bool(self.get(Website.home.related_end))

    def is_internal_content(self, language = None):
        """Indicates if the object represents content from this site.
        @rtype: bool
        """
        return True

    def is_current(self):
        now = datetime.now()
        return (self.start_date is None or self.start_date <= now) \
            and (self.end_date is None or self.end_date > now)

    def is_published(self, language = None, website = None):

        if self.per_language_publication:

            language = require_language(language)
            if language not in self.enabled_translations:
                return False

            from woost.models import Configuration
            if not Configuration.instance.language_is_enabled(language):
                return False

        elif not self.enabled:
            return False

        if not self.is_current():
            return False

        websites_subset = self.websites
        if websites_subset and website != "any":
            if website is None:
                website = get_current_website()
            if website is None or website not in websites_subset:
                return False

        return True

    def is_accessible(self, user = None, language = None, website = None):

        if user is None:
            user = get_current_user()

        return (
            self.is_published(language, website)
            and user.has_permission(
                ReadPermission,
                target = self
            )
            and (
                not self.per_language_publication
                or user.has_permission(
                    ReadTranslationPermission,
                    language = require_language(language)
                )
            )
        )

    @classmethod
    def select_published(cls, *args, **kwargs):
        return cls.select(filters = [
            IsPublishedExpression()
        ]).select(*args, **kwargs)

    @classmethod
    def select_accessible(cls, *args, **kwargs):
        return cls.select(filters = [
            IsAccessibleExpression(get_current_user())
        ]).select(*args, **kwargs)

    def get_uri(self,
        path = None,
        parameters = None,
        language = None,
        host = None,
        encode = True):

        uri = app.url_resolver.get_path(self, language = language)

        if uri is not None:
            if self.per_language_publication:
                uri = app.language.translate_uri(
                    path = uri,
                    language = require_language(language)
                )

            if path:
                uri = make_uri(uri, *path)

            if parameters:
                uri = make_uri(uri, **parameters)

            if host == "?":
                websites = self.websites
                if websites and get_current_website() not in websites:
                    host = websites[0].hosts[0]
                else:
                    host = None
            elif host == "!":
                if self.websites:
                    host = self.websites[0].hosts[0]
                else:
                    from woost.models import Configuration
                    website = (
                        get_current_website()
                        or Configuration.instance.websites[0]
                    )
                    host = website.hosts[0]

            uri = self._fix_uri(uri, host, encode)

        return uri

    def translate_file_type(self, language = None):

        trans = ""

        mime_type = self.mime_type
        if mime_type:
            trans = translations("mime " + mime_type, language = language)

        if not trans:

            res_type = self.resource_type
            if res_type:
                trans = self.__class__.resource_type.translate_value(
                    res_type,
                    language = language
                )

                if trans and res_type != "other":
                    ext = self.file_extension
                    if ext:
                        trans += " " + ext.upper().lstrip(".")

        return trans

    def get_cache_expiration(self):
        now = datetime.now()

        start = self.start_date
        if start is not None and start > now:
            return start

        end = self.end_date
        if end is not None and end > now:
            return end


Publishable.login_page.type = Publishable
Publishable.related_end = schema.Collection()


class IsPublishedExpression(Expression):
    """An expression that tests if items are published."""

    def eval(self, context, accessor = None):
        return context.is_published()

    def resolve_filter(self, query):

        def impl(dataset):

            per_lang_pub_index = Publishable.per_language_publication.index

            # Exclude disabled items
            enabled_subset = set(per_lang_pub_index.values(key = False))
            enabled_subset.intersection_update(
                Publishable.enabled.index.values(key = True)
            )

            language = get_language()

            from woost.models import Configuration
            if Configuration.instance.language_is_enabled(language):
                enabled_for_current_language = set(
                    Publishable.per_language_publication.index.values(
                        key = True
                    )
                )
                enabled_for_current_language.intersection_update(
                    Publishable.enabled_translations.index.values(
                        key = language
                    )
                )
                enabled_subset.update(enabled_for_current_language)

            dataset.intersection_update(enabled_subset)

            # Exclude content by website:
            if len(Configuration.instance.websites) > 1:

                website = get_current_website()
                per_website_index = Publishable.per_website_publication_index

                # - content that can be published on any website
                website_subset = set(per_website_index.values(key = None))

                # - content that can be published on the active website
                if website:
                    website_subset.update(per_website_index.values(key = website.id))

                dataset.intersection_update(website_subset)

            # Remove items outside their publication window
            now = datetime.now()
            dataset.difference_update(
                Publishable.start_date.index.values(
                    min = Publishable.start_date.get_index_value(now),
                    exclude_min = True
                )
            )
            dataset.difference_update(
                Publishable.end_date.index.values(
                    min = None,
                    exclude_min = True,
                    max = Publishable.end_date.get_index_value(now),
                    exclude_max = True
                )
            )

            return dataset

        return ((-1, 1), impl)


class IsAccessibleExpression(Expression):
    """An expression that tests that items can be accessed by a user.

    The expression checks both the publication state of the item and the
    read permissions for the specified user.

    @ivar user: The user that accesses the items.
    @type user: L{User<woost.models.user.User>}
    """
    def __init__(self, user = None):
        Expression.__init__(self)
        self.user = user

    def eval(self, context, accessor = None):
        return context.is_accessible(user = self.user)

    def resolve_filter(self, query):

        def impl(dataset):
            user = self.user or get_current_user()
            access_expr = PermissionExpression(user, ReadPermission)
            published_expr = IsPublishedExpression()
            dataset = access_expr.resolve_filter(query)[1](dataset)
            dataset = published_expr.resolve_filter(query)[1](dataset)

            if not user.has_permission(
                ReadTranslationPermission,
                language = require_language()
            ):
                dataset.difference_update(
                    Publishable.per_language_publication.index.values(
                        key = True
                    )
                )

            return dataset

        return ((-1, 1), impl)


class UserHasAccessLevelExpression(Expression):
    """An expression used in resolving the restrictions imposed by the
    `Publishable.access_level` member.

    The expression determines wether the indicated user belongs to one or more
    roles that have been authorized to access the access level assigned to the
    evaluated content.

    @ivar user: The user to test; defaults to the active user.
    @type user: L{User<woost.models.user.User>}
    """
    def __init__(self, user = None):
        Expression.__init__(self)
        self.user = user

    def eval(self, context, accessor = None):

        if accessor is None:
            accessor = schema.get_accessor(context)

        access_level = accessor.get(context, "access_level")

        return access_level is None or any(
            (role in access_level.roles_with_access)
            for role in (self.user or get_current_user()).iter_roles()
        )

    def resolve_filter(self, query):

        def impl(dataset):
            user = self.user or get_current_user()
            index = Publishable.access_level.index
            restricted_content = set(index.values(
                min = None,
                exclude_min = True
            ))

            for role in user.iter_roles():
                for access_level in role.access_levels:
                    restricted_content.difference_update(
                        index.values(key = access_level.id)
                    )

            dataset.difference_update(restricted_content)
            return dataset

        return ((-1, 1), impl)

# Create a single instance of the expression, to avoid instantiating it on
# every single permission check
user_has_access_level = UserHasAccessLevelExpression()


mime_type_categories = {}

for category, mime_types in (
    ("text/plain", ("text",)),
    ("html_resource", (
        "text/css",
        "text/javascript",
        "text/ecmascript",
        "application/javascript",
        "application/ecmascript"
    )),
    ("document", (
        "application/vnd.oasis.opendocument.text",
        "application/vnd.oasis.opendocument.spreadsheet",
        "application/vnd.oasis.opendocument.presentation",
        "application/msword",
        "application/msexcel",
        "application/msaccess",
        "application/mspowerpoint",
        "application/mswrite",
        "application/vnd.ms-excel",
        "application/vnd.ms-access",
        "application/vnd.ms-powerpoint",
        "application/vnd.ms-project",
        "application/vnd.ms-works",
        "application/vnd.ms-xpsdocument",
        "application/rtf",
        "application/pdf",
        "application/x-pdf",
        "application/postscript",
        "application/x-latex",
        "application/vnd.oasis.opendocument.database"
    )),
    ("package", (
        "application/zip",
        "application/x-rar-compressed",
        "application/x-tar",
        "application/x-gtar",
        "application/x-gzip",
        "application/x-bzip",
        "application/x-stuffit",
        "vnd.ms-cab-compressed"
    ))
):
    for mime_type in mime_types:
        mime_type_categories[mime_type] = category

def get_category_from_mime_type(mime_type):
    """Obtains the file category that best matches the indicated MIME type.

    @param mime_type: The MIME type to get the category for.
    @type mime_type: str

    @return: A string identifier with the category matching the indicated
        MIME type (one of 'document', 'image', 'audio', 'video', 'package',
        'html_resource' or 'other').
    @rtype: str
    """
    pos = mime_type.find("/")

    if pos != -1:
        prefix = mime_type[:pos]

        if prefix in ("image", "audio", "video"):
            return prefix

    return mime_type_categories.get(mime_type, "other")




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail.translations import translations
from cocktail import schema
from tpv.sis import SISPaymentGateway as Implementation
from woost.models import Document
from woost.extensions.payments.paymentgateway import PaymentGateway


class SISPaymentGateway(PaymentGateway, Implementation):

    instantiable = True

    default_label = schema.DynamicDefault(
        lambda: translations("SISPaymentGateway.label default")
    )

    members_order = [
        "merchant_name",
        "merchant_code",
        "merchant_terminal",
        "merchant_secret_key",
        "pay_methods",
        "payment_successful_page",
        "payment_failed_page"
    ]

    merchant_code = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    merchant_name = schema.String(
        shadows_attribute = True,
        text_search = False
    )

    merchant_terminal = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    merchant_secret_key = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    pay_methods = schema.Collection(
        shadows_attribute = True,
        items = schema.String(
            enumeration = ["T", "D", "R"],
            translate_value = lambda value, language = None, **kwargs: \
                translations("SISPaymentGateway.pay_methods=%s" % (value,))
        ),
        edit_control = "cocktail.html.CheckList"
    )

    payment_successful_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    payment_failed_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    def get_payment_successful_url(self, payment):
        if self.payment_successful_page:
            return self.payment_successful_page.get_uri(
                host = "!",
                parameters = {"payment_id": payment.id}
            )

    def get_payment_failed_url(self, payment):
        if self.payment_failed_page:
            return self.payment_failed_page.get_uri(
                host = "!",
                parameters = {"payment_id": payment.id}
            )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from datetime import datetime
import cherrypy
from createsend import CreateSend, Client, List, Subscriber, BadRequest
from cocktail import schema
from cocktail.controllers import Controller, request_property
from cocktail.controllers.parameters import get_parameter
from woost.models import Configuration
from woost.extensions.campaign3.campaignmonitorlist import CampaignMonitorList


class ResubscribeController(Controller):

    max_seconds = 60 * 60
    resubscribed_lists = None

    def __init__(self):
        self.resubscribed_lists = []

    @request_property
    def email(self):
        return get_parameter(schema.String("email"))

    def resubscribe(self):
        cs = CreateSend(
            {"api_key": Configuration.instance.get_setting("campaign_monitor_api_key")}
        )
        now = datetime.now()

        for client in cs.clients():
            client = Client(
                {"api_key": Configuration.instance.get_setting("campaign_monitor_api_key")},
                client.ClientID
            )
            for list in client.lists():
                subscriber = Subscriber(
                    {"api_key": Configuration.instance.get_setting("campaign_monitor_api_key")}
                )
                try:
                    response = subscriber.get(list.ListID, self.email)
                except BadRequest:
                    continue

                date = datetime.strptime(response.Date, '%Y-%m-%d %H:%M:%S')
                diff = now - date
                if response.State != "Active" and (
                    date > now or diff.seconds < self.max_seconds
                ):
                    response = subscriber.add(
                        list.ListID,
                        self.email,
                        response.Name,
                        None,
                        True
                    )
                    self.resubscribed_lists.append(list.ListID)

    @request_property
    def action(self):
        return get_parameter("action")

    @request_property
    def submitted(self):
        return cherrypy.request.method == "POST" \
            and self.action == "resubscribe" \
            and self.email

    def submit(self):
        self.resubscribe()
        self.after_submit()

    def after_submit(self):
        # Redirect the user to the confirmation page
        for list_id in self.resubscribed_lists:
            list = CampaignMonitorList.get_instance(list_id = list_id)
            if list \
            and list.confirmation_page \
            and list.confirmation_page.is_accessible():
                raise cherrypy.HTTPRedirect(list.confirmation_page.get_uri())

    @request_property
    def output(self):
        output = Controller.output(self)
        output["email"] = self.email
        return output



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Slot, Block, ElementType


class OrderStepsContainerBlock(Block):

    instantiable = True
    type_group = "blocks.ecommerce"
    view_class = "woost.extensions.ecommerce.OrderStepsContainerBlockView"

    members_order = [
        "list_type"
    ]

    list_type = schema.String(
        required = True,
        default = "div",
        enumeration = [
            "div",
            "ul",
            "ol",
            "dl"
        ],
        member_group = "content"
    )

    blocks = Slot()

    def init_view(self, view):
        Block.init_view(self, view)
        view.step_content.tag = self.list_type
        view.blocks = self.blocks
        return view




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.stringutils import normalize
from cocktail.translations import translations
from cocktail import schema


class LocaleMember(schema.String):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("search_control", "cocktail.html.DropdownSelector")
        kwargs.setdefault("enumeration", _locales_enumeration)
        schema.String.__init__(self, *args, **kwargs)

    def translate_value(self, value, language = None, **kwargs):
        if not value:
            return schema.String.translate_value(
                self,
                value,
                language = language,
                **kwargs
            )
        else:
            return translations(
                "locale",
                locale = value,
                language = language,
                **kwargs
            )

def _locales_enumeration(ctx):
    from woost.models.configuration import Configuration
    return sorted(
        Configuration.instance.languages,
        key = lambda locale: normalize(translations("locale", locale = locale))
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.events import when
from cocktail import schema
from cocktail.controllers import request_property
from woost.models import Publishable, VideoPlayerSettings
from .block import Block
from .elementtype import ElementType


class VideoBlock(Block):

    instantiable = True
    block_display = "woost.views.VideoBlockDisplay"

    member_order = ["element_type", "video", "player_settings"]

    element_type = ElementType(
        member_group = "content"
    )

    video = schema.Reference(
        type = Publishable,
        required = True,
        relation_constraints = {"resource_type": "video"},
        related_end = schema.Collection(),
        invalidates_cache = True,
        member_group = "content"
    )

    player_settings = schema.Reference(
        type = VideoPlayerSettings,
        required = True,
        related_end = schema.Collection(),
        invalidates_cache = True,
        member_group = "content"
    )

    view_class = schema.String(
        required = True,
        shadows_attribute = True,
        enumeration = [
            "woost.views.VideoBlockView",
            "woost.views.VideoPopUp"
        ],
        default = "woost.views.VideoBlockView",
        edit_control = "cocktail.html.RadioSelector",
        member_group = "content"
    )


    def get_block_image(self):
        return self.video

    def init_view(self, view):
        Block.init_view(self, view)
        view.tag = self.element_type
        view.video = self.video
        view.player_settings = self.player_settings
        view.depends_on(self.video)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from cocktail.translations import translations
from woost.models import Configuration, Item
from createsend import CreateSend, List


class CampaignMonitorList(Item):

    edit_node_class = \
        "woost.extensions.campaign3.campaignmonitorlisteditnode.CampaignMonitorListEditNode"

    members_order = [
        "title",
        "list_id",
        "confirmation_page",
        "confirmation_success_page",
        "unsubscribe_page",
    ]

    title = schema.String(
        required = True,
        descriptive = True
    )

    list_id = schema.String(
        required = True,
        unique = True,
        text_search = False
    )

    confirmation_page = schema.Reference(
        type = "woost.models.Publishable",
        related_end = schema.Collection()
    )

    confirmation_success_page = schema.Reference(
        type = "woost.models.Publishable",
        related_end = schema.Collection()
    )

    unsubscribe_page = schema.Reference(
        type = "woost.models.Publishable",
        related_end = schema.Collection()
    )

    def update(self):
        list = List(
            {"api_key": Configuration.instance.get_setting("campaign_monitor_api_key")},
            self.list_id
        )
        details = list.details()

        if self.confirmation_success_page:
            confirmation_success_page = self.confirmation_success_page.get_uri(
                host = "."
            )
        else:
            confirmation_success_page = None

        if self.unsubscribe_page:
            unsubscribe_page = self.unsubscribe_page.get_uri(host = ".") + "?email=[email]"
        else:
            unsubscribe_page = None

        list.update(
            details.Title,
            unsubscribe_page,
            details.ConfirmedOptIn,
            confirmation_success_page,
            unsubscribe_setting=details.UnsubscribeSetting
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension, Controller, Configuration
from woost.models.rendering import ChainRenderer


translations.define("IssuuExtension",
    ca = u"Documents Issuu",
    es = u"Documentos Issuu",
    en = u"Issuu documents"
)

translations.define("IssuuExtension-plural",
    ca = u"Documents Issuu",
    es = u"Documentos Issuu",
    en = u"Issuu documents"
)


class IssuuExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afegeix suport per documents Issuu""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para documentos Issuu""",
            "es"
        )
        self.set("description",
            u"""Adds support for Issuu documents""",
            "en"
        )

    def _load(self):
        from woost.extensions.issuu import (
            strings,
            issuudocument,
            issuublock,
        )
        self.install()
        self.register_view_factory()

    def _install(self):

        # Create the product controller
        controller = Controller()
        controller.qname = "woost.issuu_document_controller"
        for language in Configuration.instance.languages:
            value = translations(
                "woost.extensions.issuu.issuu_document_controller.title",
                language
            )
            if value:
                controller.set("title", value, language)
        controller.python_name = \
            "woost.extensions.issuu.issuudocumentcontroller.IssuuDocumentController"
        controller.insert()

        self._create_renderers()

    def _create_renderers(self):

        from woost.extensions.issuu.issuudocumentrenderer \
            import IssuuDocumentRenderer

        # Look for the first chain renderer
        for renderer in Configuration.instance.renderers:
            if isinstance(renderer, ChainRenderer):

                # Add the renderer for Issuu documents
                issuu_renderer = IssuuDocumentRenderer()
                issuu_renderer.insert()
                renderer.renderers.append(issuu_renderer)

                break

    def register_view_factory(self):

        from cocktail.html import templates
        from woost.extensions.issuu.issuudocument import IssuuDocument
        from woost.views.viewfactory import publishable_view_factory

        def issuu_viewer(item, parameters):
            viewer = templates.new("cocktail.html.IssuuViewer")
            viewer.config_id = item.issuu_config_id
            viewer.page_number = item.thumbnail_page
            return viewer

        publishable_view_factory.register_first(
            IssuuDocument,
            "issuu_viewer",
            issuu_viewer
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import re
from warnings import warn
from decimal import Decimal
from cocktail.modeling import classgetter
from cocktail.translations import (
    translations,
    require_language,
    set_language,
    set_fallback_languages
)
from cocktail import schema
from woost import app
from .item import Item
from .slot import Slot
from .publishable import Publishable
from .website import Website
from .websitesession import get_current_website
from .caching import CachingPolicy
from .rendering.renderer import Renderer
from .rendering.imagefactory import ImageFactory
from .videoplayersettings import VideoPlayerSettings
from .trigger import Trigger

try:
    from fractions import Fraction
except ImportError:
    _numerical_types = (int, float, Decimal)
else:
    _numerical_types = (int, float, Decimal, Fraction)


class Configuration(Item):

    instantiable = False
    type_group = "setup"
    skip_listing = True

    groups_order = [
        "publication",
        "publication.pages",
        "publication.maintenance",
        "language",
        "media.images",
        "media.video",
        "rendering",
        "services",
        "system",
        "system.smtp",
        "admin"
    ]

    members_order = [
        "websites",
        "caching_policies",
        "down_for_maintenance",
        "maintenance_page",
        "maintenance_addresses",
        "login_page",
        "generic_error_page",
        "not_found_error_page",
        "forbidden_error_page",
        "languages",
        "published_languages",
        "virtual_languages",
        "default_language",
        "fallback_languages",
        "heed_client_language",
        "backoffice_language",
        "backoffice_language_chain",
        "renderers",
        "image_factories",
        "video_player_settings",
        "timezone",
        "smtp_host",
        "smtp_user",
        "smtp_password",
        "triggers"
    ]

    @classgetter
    def instance(cls):
        return cls.get_instance(qname = "woost.configuration")

    common_blocks = Slot()

    # publication
    #--------------------------------------------------------------------------
    websites = schema.Collection(
        items = schema.Reference(type = Website),
        related_end = schema.Collection(),
        integral = True,
        member_group = "publication"
    )

    caching_policies = schema.Collection(
        items = schema.Reference(type = CachingPolicy),
        integral = True,
        related_end = schema.Reference(),
        member_group = "publication"
    )

    login_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    generic_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    not_found_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    forbidden_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    down_for_maintenance = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "publication.maintenance"
    )

    maintenance_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "publication.maintenance"
    )

    maintenance_addresses = schema.Collection(
        items = schema.String(),
        listed_by_default = False,
        searchable = False,
        member_group = "publication.maintenance"
    )

    # language
    #------------------------------------------------------------------------------
    languages = schema.Collection(
        items = schema.String(
            format = "^[a-z]{2}(-[A-Z]{2})?(-[a-z]+)?$"
        ),
        min = 1,
        listed_by_default = False,
        searchable = False,
        member_group = "language"
    )

    published_languages = schema.Collection(
        items = schema.String(
            enumeration = languages,
            edit_control = "cocktail.html.TextBox"
        ),
        searchable = False,
        listed_by_default = False,
        member_group = "language"
    )

    virtual_languages = schema.Collection(
        items = schema.String(
            enumeration = languages,
            edit_control = "cocktail.html.TextBox"
        ),
        searchable = False,
        listed_by_default = False,
        member_group = "language"
    )

    default_language = schema.String(
        required = True,
        enumeration = languages,
        edit_control = "cocktail.html.TextBox",
        text_search = False,
        listed_by_default = False,
        searchable = False,
        member_group = "language"
    )

    fallback_languages = schema.Collection(
        items = (
            schema.Tuple(
                items = (
                    schema.String(enumeration = languages),
                    schema.Collection(
                        type = tuple,
                        items = schema.String(
                            enumeration = languages
                        ),
                        request_value_separator = ","
                    )
                ),
                request_value_separator = ":"
            )
        ),
        request_value_separator = "\n",
        edit_control = "cocktail.html.TextArea",
        searchable = False,
        member_group = "language"
    )

    def setup_languages(self):
        set_language(self.default_language)
        for language, fallback_languages in self.fallback_languages:
            set_fallback_languages(language, fallback_languages)

    heed_client_language = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "language"
    )

    backoffice_language = schema.String(
        required = True,
        enumeration = ["en", "es", "ca"],
        default = "en",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(value, language, **kwargs),
        listed_by_default = False,
        member_group = "language"
    )

    backoffice_language_chain = schema.Collection(
        items = schema.String(
            enumeration = languages,
            edit_control = "cocktail.html.TextBox"
        ),
        searchable = False,
        listed_by_default = False,
        member_group = "language"
    )

    # media
    #--------------------------------------------------------------------------
    renderers = schema.Collection(
        items = schema.Reference(type = Renderer),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "media.images"
    )

    image_factories = schema.Collection(
        items = schema.Reference(type = ImageFactory),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "media.images"
    )

    video_player_settings = schema.Collection(
        items = schema.Reference(type = VideoPlayerSettings),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "media.video"
    )

    # system
    #--------------------------------------------------------------------------
    timezone = schema.String(
        required = False,
        format = re.compile(r'^["GMT"|"UTC"|"PST"|"MST"|"CST"|"EST"]{3}$|^[+-]\d{4}$'),
        text_search = False,
        member_group = "system",
        listed_by_default = False
    )

    smtp_host = schema.String(
        default = "localhost",
        listed_by_default = False,
        text_search = False,
        member_group = "system.smtp"
    )

    smtp_user = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "system.smtp"
    )

    smtp_password = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "system.smtp"
    )

    # administration
    #--------------------------------------------------------------------------
    triggers = schema.Collection(
        items = schema.Reference(type = Trigger),
        related_end = schema.Reference(),
        integral = True,
        member_group = "administration"
    )

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.__name__, language, **kwargs)

    def resolve_path(self, path):
        warn(
            "Configuration.resolve_path is deprecated, use app.url_resolver.resolve_path instead",
            DeprecationWarning,
            stacklevel = 2
        )
        return app.url_resolver.resolve_path(path)

    def get_path(self, publishable, language = None):
        warn(
            "Configuration.get_path is deprecated, use app.url_resolver.get_path instead",
            DeprecationWarning,
            stacklevel = 2
        )
        return app.url_resolver.get_path(publishable, language = language)

    def get_setting(self, key):
        """Obtains the value for the indicated configuration option.

        The method will search the active website first (see
        L{woost.models.get_current_website}), and the site's L{Configuration}
        if there is no active website, or if the website provides no
        significant value. Significant values include numbers, booleans, and
        any object that evaluates to True on a boolean context.

        @param key: The name of the value to obtain. Must match one of the
            members of the L{Configuration} or the L{Website} models.
        @type key: str

        @return: The value for the indicated configuration option.
        """
        website = get_current_website()

        if website is not None:
            value = getattr(website, key, None)
            if self._is_significant_setting_value(key, value):
                return value

        return getattr(self.instance, key, None)

    def _is_significant_setting_value(self, key, value):
        return value or isinstance(value, _numerical_types)

    def language_is_enabled(self, language = None):
        language_subset = self.get_setting("published_languages")
        if not language_subset:
            return True
        else:
            return require_language(language) in language_subset

    def get_enabled_languages(self):
        return self.get_setting("published_languages") or self.languages

    def connect_to_smtp(self):

        import smtplib
        smtp = smtplib.SMTP(self.get_setting("smtp_host"), smtplib.SMTP_PORT)

        user = self.get_setting("smtp_user")
        password = self.get_setting("smtp_password")

        if user and password:
            smtp.login(str(user), str(password))

        return smtp

    def get_website_by_host(self, host):

        # Specific match
        for website in self.websites:
            if host in website.hosts:
                return website

        # Partial wildcards (ie. *.foo.com)
        while True:
            pos = host.find(".")

            if pos == -1:
                break

            host = host[pos + 1:]
            wildcard = "*." + host
            for website in self.websites:
                if wildcard in website.hosts:
                    return website

        # Generic wildcard (*)
        for website in self.websites:
            if "*" in website.hosts:
                return website




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Configuration
from woost.extensions.shorturls.urlshortener import URLShortener


Configuration.add_member(
    schema.Reference("url_shortener",
        type = URLShortener,
        related_end = schema.Reference(),
        member_group = "services.short_urls",
        after_member = "caching_policies",
        listed_by_default = False
    )
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .block import Block


class HTMLBlock(Block):

    instantiable = True
    type_group = "blocks.custom"
    view_class = "cocktail.html.Element"

    html = schema.CodeBlock(
        language = "html",
        member_group = "content"
    )

    translated_html = schema.CodeBlock(
        translated = True,
        language = "html",
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        html = self.translated_html or self.html
        if html:
            view.append(html)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.pkgutils import resolve
from woost.controllers.basecmscontroller import BaseCMSController
from woost.extensions.payments.paymentgateway import PaymentGateway


class PaymentRootController(BaseCMSController):
    """Root controller for all payment related requests.

    This controller does nothing by itself, it merely identifies the payment
    gateway that should handle an incomming request and forwards the request to
    the gateway's controller (an instance of L{PaymentGatewayController}).
    """

    def resolve(self, path):

        if not path:
            raise cherrypy.NotFound()

        # Identify the gateway involved in the payment
        gateway_id = path.pop(0)

        try:
            gateway_id = int(gateway_id)
        except:
            raise cherrypy.HTTPError(400)

        gateway = PaymentGateway.get_instance(gateway_id)

        if gateway is None:
            raise cherrypy.NotFound()

        # Forward the request to the gateway's controller
        controller_class = resolve(gateway.payment_gateway_controller_class)
        return controller_class(gateway)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Publishable
from .block import Block


class TweetButton(Block):

    instantiable = True
    type_group = "blocks.social"
    view_class = "woost.views.TweetButton"

    groups_order = list(Block.groups_order)
    pos = groups_order.index("content")
    groups_order = (
        groups_order[:pos + 1]
        + ["tweet", "appearence"]
        + groups_order[pos + 1:]
    )
    del pos

    members_order = [
        "tw_target",
        "tw_via",
        "tw_related",
        "tw_hashtags",
        "tw_text",
        "tw_dnt",
        "tw_size",
        "tw_count"
    ]

    tw_target = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "tweet"
    )

    tw_via = schema.String(
        member_group = "tweet"
    )

    tw_related = schema.String(
        member_group = "tweet"
    )

    tw_hashtags = schema.String(
        member_group = "tweet"
    )

    tw_text = schema.String(
        translated = True,
        edit_control = "cocktail.html.TextArea",
        member_group = "tweet"
    )

    tw_dnt = schema.Boolean(
        required = True,
        default = False,
        member_group = "tweet"
    )

    tw_size = schema.String(
        required = True,
        default = "medium",
        enumeration = ["medium", "big"],
        edit_control = "cocktail.html.RadioSelector",
        member_group = "appearence"
    )

    tw_count = schema.String(
        required = True,
        default = "horizontal",
        enumeration = ["none", "horizontal", "vertical"],
        edit_control = "cocktail.html.RadioSelector",
        member_group = "appearence"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.url = self.tw_target and self.tw_target.get_uri(host = ".")
        view.via = self.tw_via
        view.related = self.tw_related
        view.hashtags = self.tw_hashtags
        view.dnt = self.tw_dnt
        view.text = self.tw_text
        view.size = self.tw_size
        view.count = self.tw_count




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

_undefined = object()


class TypeGroup(object):

    def __init__(self, id, children = None):
        self.id = id
        self.__children = []
        if children is not None:
            for child in children:
                self.append(child)

    def __repr__(self):
        return "TypeGroup(%r)" % self.id

    def __translate__(self, language, **kwargs):
        return translations("woost.type_groups." + self.id, language, **kwargs)

    def append(self, child):
        self.__children.append(child)

    def insert(self, position, child):

        # Insert after / before an existing child
        if isinstance(position, basestring):
            parts = position.split()
            if len(parts) == 2:
                for index, existing_child in enumerate(self.__children):
                    if existing_child.id == parts[1]:
                        break
                if parts[0] == "before":
                    position = index
                elif parts[0] == "after":
                    position = index + 1

        if not isinstance(position, int):
            raise ValueError("Invalid position: %s" % position)

        self.__children.insert(position, child)

    def pop(self, child_id, default = _undefined):
        child = None

        for index, existing_child in enumerate(self.__children):
            if existing_child.id == child_id:
                child = existing_child
                break

        if child is not None:
            del self.__children[index]
            return child

        if default != _undefined:
            return default

        raise KeyError("Unknown type group: " + child_id)


    def remove(self, child):
        if isinstance(child, basestring):
            self.pop(child)
        else:
            self.__children.remove(child)

    def __iter__(self):
        return iter(self.__children)

    def __len__(self):
        return len(self.__children)

    def __contains__(self, child):
        if isinstance(child, basestring):
            child_id = child
            return any(child.id == child_id for child in self.__children)
        else:
            return child in self.__children

    def __getitem__(self, child_id):

        for child in self.__children:
            if child.id == child_id:
                return child

        raise KeyError("Unknown type group: " + child_id)

    def __delitem__(self, child):
        self.remove(child)

type_groups = TypeGroup("global", [
    TypeGroup("publishable", [
        TypeGroup("document"),
        TypeGroup("resource")
    ]),
    TypeGroup("setup", [
        TypeGroup("customization"),
        TypeGroup("users")
    ]),
    TypeGroup("misc")
])

block_type_groups = TypeGroup("blocks", [
    TypeGroup("blocks.content"),
    TypeGroup("blocks.layout"),
    TypeGroup("blocks.listings"),
    TypeGroup("blocks.social"),
    TypeGroup("blocks.forms"),
    TypeGroup("blocks.custom")
])




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Item


class AccessLevel(Item):

    type_group = "users"

    members_order = [
        "title",
        "roles_with_access",
        "restricted_content"
    ]

    title = schema.String(
        translated = True,
        unique = True,
        indexed = True,
        normalized_index = False
    )

    roles_with_access = schema.Collection(
        items = "woost.models.Role",
        bidirectional = True
    )

    restricted_content = schema.Collection(
        items = "woost.models.Publishable",
        bidirectional = True,
        editable = False
    )

    def __translate__(self, language, **kwargs):

        if self.title:
            return self.title

        if self.roles_with_access:
            return translations(
                "woost.models.accesslevel.AccessLevel-instance",
                instance = self
            )

        return Item.__translate__(self, language, **kwargs)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.html.datadisplay import display_factory
from cocktail.iteration import first
from .configuration import Configuration
from .rendering import ImageFactory

def _iter_block_image_factories():
    for factory in Configuration.instance.image_factories:
        if factory.applicable_to_blocks:
            yield factory

def _block_image_factories_enumeration(ctx):
    return list(_iter_block_image_factories())

_block_image_factories_default = schema.DynamicDefault(
    lambda: first(_iter_block_image_factories())
)

_mandatory_dropdown = display_factory(
    "cocktail.html.DropdownSelector",
    empty_option_displayed = False
)


class BlockImageFactoryReference(schema.Reference):

    def __init__(self, *args, **kwargs):

        kwargs.setdefault("required", True)
        kwargs.setdefault("type", ImageFactory)
        kwargs.setdefault("enumeration", _block_image_factories_enumeration)
        kwargs.setdefault("default", _block_image_factories_default)
        kwargs.setdefault("edit_control", _mandatory_dropdown)

        if "bidirectional" not in kwargs and "related_end" not in kwargs:
            kwargs["related_end"] = schema.Collection()

        schema.Reference.__init__(self, *args, **kwargs)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.events import event_handler
from cocktail import schema
from cocktail.controllers import (
    FormProcessor,
    Form,
    request_property,
    serve_file
)
from cocktail.persistence import transactional
from woost.models import (
    Synchronization,
    Item,
    User,
    File,
    get_current_user,
    InstallationSyncPermission
)
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class SiteSyncController(FormProcessor, BaseBackOfficeController):

    @request_property
    def synchronization(self):
        return Synchronization()

    @event_handler
    def handle_traversed(cls, e):
        get_current_user().require_permission(InstallationSyncPermission)

    @cherrypy.expose
    @transactional()
    def manifest(self):
        cherrypy.response.headers["Content-Type"] = "text/plain"
        for global_id, object_hash in self.synchronization.process_manifest():
            yield "%s %s\n" % (global_id, object_hash)

    @cherrypy.expose
    def state(self, identifiers):
        cherrypy.response.headers["Content-Type"] = "application/json"

        glue = ""
        yield "{"

        for global_id in identifiers.split(","):
            obj = Item.require_instance(global_id = global_id)
            yield '%s"%s":' % (glue, global_id)
            yield self.synchronization.serialize_object_state(obj)
            glue = ","

        yield "}"

    @cherrypy.expose
    def file(self, global_id):
        file = File.require_instance(global_id = global_id)
        return serve_file(
            file.file_path,
            name = file.file_name,
            content_type = file.mime_type
        )

    class SyncForm(Form):

        model = schema.Schema("SiteSync", members = [
            schema.URL("site_url",
                required = True
            ),
            schema.String("remote_user",
                required = True
            ),
            schema.String("remote_password",
                required = True,
                edit_control = "cocktail.html.PasswordBox"
            )
        ])

        def submit(self):
            Form.submit(self)
            pass




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2014
"""
from cocktail.modeling import cached_getter
from cocktail.stringutils import normalize
from cocktail.translations import translations, descend_language_tree
from cocktail.html.datadisplay import display_factory
from woost.models import Configuration
from woost.models.enabledtranslations import (
    auto_enable_translations,
    auto_disable_translations
)
from woost.controllers.backoffice.editstack import EditNode


class EnabledTranslationsEditNode(EditNode):

    @cached_getter
    def form_schema(self):
        form_schema = EditNode.form_schema(self)

        enabled_translations = form_schema.get_member("enabled_translations")
        if enabled_translations is not None:
            enabled_translations.items.enumeration = \
                lambda ctx: self._enabled_translations_enumeration(ctx)
            enabled_translations.edit_control = display_factory(
                "woost.views.EnabledTranslationsSelector",
                defined_translations = self.item_translations
            )

        return form_schema

    def _enabled_translations_enumeration(self, ctx):
        eligible_languages = set()

        for language in self.item_translations:
            eligible_languages.update(descend_language_tree(language))

        eligible_languages.difference_update(
            Configuration.instance.virtual_languages
        )

        return sorted(
            list(eligible_languages),
            key = lambda language:
                normalize(translations("locale", locale = language))
        )

    def add_translation(self, language):
        EditNode.add_translation(self, language)
        enabled_translations = self.form_data.get("enabled_translations")
        if enabled_translations is not None:
            auto_enable_translations(
                language,
                self.item_translations,
                enabled_translations
            )

    def remove_translation(self, language):
        EditNode.remove_translation(self, language)
        enabled_translations = self.form_data.get("enabled_translations")
        if enabled_translations is not None:
            auto_disable_translations(
                language,
                self.item_translations,
                enabled_translations
            )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import VideoPlayerSettings
from woost.extensions.youtube.youtubevideo import YouTubeVideo


VideoPlayerSettings.members_order = VideoPlayerSettings.members_order + [
        "youtube_allow_fullscreen",
        "youtube_show_info",
        "youtube_show_related_videos"
    ]

VideoPlayerSettings.add_member(
    schema.Boolean(
        "youtube_allow_fullscreen",
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "youtube"
    )
)

VideoPlayerSettings.add_member(
    schema.Boolean(
        "youtube_show_info",
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "youtube"
    )
)

VideoPlayerSettings.add_member(
    schema.Boolean(
        "youtube_show_related_videos",
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "youtube"
    )
)


def player_initialization(player, settings, video):
    player.video_id = video.video_id
    player.width = settings.width
    player.height = settings.height
    player.autoplay = settings.autoplay
    player.show_player_controls = settings.show_player_controls
    player.allow_fullscreen = settings.youtube_allow_fullscreen
    player.show_info = settings.youtube_show_info
    player.show_related_videos = settings.youtube_show_related_videos

VideoPlayerSettings.player_initialization[YouTubeVideo] = player_initialization




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail import schema
from .block import Block
from .publishable import Publishable


class LoginBlock(Block):

    instantiable = True
    type_group = "blocks.forms"
    view_class = "woost.views.LoginBlockView"

    login_target = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        if self.login_target is not None:
            view["action"] = self.login_target.get_uri()
        if hasattr(cherrypy.request, "authentication_errors"):
            view.errors = cherrypy.request.authentication_errors




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from __future__ import with_statement
import sha
from string import letters, digits
from random import choice
from optparse import OptionParser
from cocktail.stringutils import random_string
from cocktail.translations import translations
from cocktail.iteration import first
from cocktail.persistence import (
    datastore,
    mark_all_migrations_as_executed,
    reset_incremental_id
)
from woost import app
from woost.models import (
    changeset_context,
    Item,
    Configuration,
    Website,
    Publishable,
    Document,
    Page,
    TextBlock,
    LoginBlock,
    CustomBlock,
    User,
    Role,
    URI,
    Controller,
    Template,
    Style,
    File,
    UserView,
    Permission,
    ReadPermission,
    CreatePermission,
    ModifyPermission,
    DeletePermission,
    RenderPermission,
    ReadMemberPermission,
    ModifyMemberPermission,
    CreateTranslationPermission,
    InstallationSyncPermission,
    ReadTranslationPermission,
    ModifyTranslationPermission,
    DeleteTranslationPermission,
    ReadHistoryPermission,
    AccessLevel,
    DeleteTrigger,
    CustomTriggerResponse,
    EmailTemplate,
    CachingPolicy,
    Extension,
    Trigger,
    TriggerResponse,
    SiteInstallation,
    VideoPlayerSettings,
    rendering,
    load_extensions
)


class TranslatedValues(object):

    def __init__(self, key = None, **kwargs):
        self.key = key
        self.kwargs = kwargs


class SiteInitializer(object):

    admin_email = "admin@localhost"
    admin_password = ""
    languages = ["en"]
    extensions = []
    hosts = ["localhost"]
    base_id = None

    read_only_types = [
        Style,
        User,
        Template,
        EmailTemplate,
        rendering.Renderer,
        rendering.ImageFactory,
        VideoPlayerSettings
    ]

    restricted_types = [
        Role,
        Controller,
        Permission,
        CachingPolicy,
        SiteInstallation,
        Trigger,
        TriggerResponse,
        UserView,
        Extension
    ]

    read_only_members = [
        "woost.models.publishable.Publishable.mime_type",
        "woost.models.configuration.Configuration.login_page",
        "woost.models.configuration.Configuration.generic_error_page",
        "woost.models.configuration.Configuration.not_found_error_page",
        "woost.models.configuration.Configuration.forbidden_error_page",
        "woost.models.configuration.Configuration.maintenance_page",
        "woost.models.website.Website.login_page",
        "woost.models.website.Website.home",
        "woost.models.website.Website.generic_error_page",
        "woost.models.website.Website.not_found_error_page",
        "woost.models.website.Website.forbidden_error_page",
        "woost.models.website.Website.maintenance_page",
    ]

    restricted_members = [
        "woost.models.item.Item.qname",
        "woost.models.item.Item.global_id",
        "woost.models.item.Item.synchronizable",
        "woost.models.publishable.Publishable.encoding",
        "woost.models.publishable.Publishable.login_page",
        "woost.models.publishable.Publishable.requires_https",
        "woost.models.configuration.Configuration.maintenance_addresses",
        "woost.models.configuration.Configuration.languages",
        "woost.models.configuration.Configuration.published_languages",
        "woost.models.configuration.Configuration.default_language",
        "woost.models.configuration.Configuration.heed_client_language",
        "woost.models.configuration.Configuration.timezone",
        "woost.models.configuration.Configuration.smtp_host",
        "woost.models.configuration.Configuration.smtp_user",
        "woost.models.configuration.Configuration.smtp_password",
        "woost.models.configuration.Configuration.renderers",
        "woost.models.configuration.Configuration.image_factories",
        "woost.models.configuration.Configuration.video_player_settings",
        "woost.models.website.Website.hosts",
        "woost.models.website.Website.https_policy",
        "woost.models.website.Website.https_persistence",
        "woost.models.website.Website.published_languages",
        "woost.models.website.Website.default_language",
        "woost.models.website.Website.heed_client_language",
        "woost.models.block.Block.initialization"
    ]

    image_factories = [
        "default",
        "icon16",
        "icon32",
        "backoffice_thumbnail",
        "backoffice_small_thumbnail",
        "edit_blocks_thumbnail",
        "close_up",
        "default_thumbnail",
    ]

    def main(self):

        parser = OptionParser()
        parser.add_option("-u", "--user", help = "Administrator email")
        parser.add_option("-p", "--password", help = "Administrator password")
        parser.add_option("-l", "--languages",
            help = "Comma separated list of languages")
        parser.add_option("-e", "--extensions",
            default = "",
            help = "The list of extensions to enable")
        parser.add_option("-b", "--base-id",
            type = int,
            help = "Seed the incremental ID sequence at a non-zero value")
        parser.add_option("-i", "--installation-id",
            default = "DEV",
            help = "A unique prefix for this installation, used to assign "
                   "globally unique identifiers for objects."
        )

        options, args = parser.parse_args()

        self.admin_email = options.user
        self.admin_password = options.password

        if self.admin_email is None:
            self.admin_email = raw_input("Administrator email: ") or "admin@localhost"

        if self.admin_password is None:
            self.admin_password = raw_input("Administrator password: ") \
                or random_string(8)

        languages = options.languages \
            and options.languages.replace(",", " ") \
            or raw_input("Languages: ") or "en"

        self.languages = languages.split()
        self.extensions = options.extensions.split(",")
        self.base_id = options.base_id
        app.installation_id = options.installation_id

        self.initialize()

        print u"Your site has been successfully created. You can start it by " \
              u"executing the 'run.py' script. An administrator account for the " \
              u"content manager interface has been generated, with the " \
              u"following credentials:\n\n" \
              u"\tEmail:     %s\n" \
              u"\tPassword:  %s\n\n" % (self.admin_email, self.admin_password)

    def initialize(self):
        self.reset_database()

        with changeset_context() as changeset:
            self.changeset = changeset
            self.create_content()

        mark_all_migrations_as_executed()
        datastore.commit()

    def reset_database(self):
        datastore.clear()
        datastore.close()

        if self.base_id:
            reset_incremental_id(self.base_id)

    def _create(self, _model, **values):
        instance = _model()

        for key, value in values.iteritems():
            if isinstance(value, TranslatedValues):
                trans_key = "woost.initialization."

                if value.key:
                    trans_key += value.key
                else:
                    qname = values.get("qname")
                    if not qname:
                        raise ValueError(
                            "Can't translate the values for %s without "
                            "giving it a qname or providing an explicit "
                            "translation key"
                            % instance
                        )
                    prefix = "woost."
                    assert qname.startswith(prefix)
                    trans_key += qname[len(prefix):] + "." + key

                for language in self.languages:
                    trans = translations(trans_key, language, **value.kwargs)
                    if trans:
                        instance.set(key, trans, language)
            else:
                setattr(instance, key, value)

        instance.insert()
        return instance

    def create_content(self):

        # Bootstrap: create the site's configuration and administrator user
        self.configuration = self.create_configuration()
        self.administrator = self.create_administrator()

        self.configuration.author = self.administrator
        self.administrator.author = self.administrator

        # From this point, the authorship f or further objects is set
        # automatically through the active changeset
        self.changeset.author = self.administrator

        # Default website
        self.website = self.create_website()
        self.configuration.websites.append(self.website)

        # Roles
        self.anonymous_role = self.create_anonymous_role()
        self.anonymous_user = self.create_anonymous_user()

        self.administrator_role = self.create_administrator_role()
        self.administrator.roles.append(self.administrator_role)

        self.everybody_role = self.create_everybody_role()
        self.authenticated_role = self.create_authenticated_role()
        self.editor_role = self.create_editor_role()

        self.editor_access_level = self.create_editor_access_level()

        # File deletion trigger
        self.file_deletion_trigger = self.create_file_deletion_trigger()
        self.configuration.triggers.append(self.file_deletion_trigger)

        # Templates and controllers
        self.create_controllers()
        self.standard_template = self.create_standard_template()

        # Home page
        self.website.home = self.create_home()

        # Default stylesheets
        self.user_stylesheet = self.create_user_stylesheet()

        # Backoffice
        self.backoffice = self.create_backoffice()

        # Error pages
        self.not_found_error_page = self.create_not_found_error_page()
        self.configuration.not_found_error_page = self.not_found_error_page

        self.forbidden_error_page = self.create_forbidden_error_page()
        self.configuration.forbidden_error_page = self.forbidden_error_page

        # Password change
        self.password_change_page = self.create_password_change_page()
        self.password_change_confirmation_page = \
            self.create_password_change_confirmation_page()
        self.password_change_email_template = \
            self.create_password_change_email_template()

        # Login page
        self.login_page = self.create_login_page()
        self.configuration.login_page = self.login_page

        # User views
        self.page_tree_user_view = self.create_page_tree_user_view()
        self.create_file_gallery_user_view()

        # Renderers
        self.content_renderer = self.create_content_renderer()
        self.icon16_renderer = self.create_icon16_renderer()
        self.icon32_renderer = self.create_icon32_renderer()

        self.configuration.renderers = [
            self.content_renderer,
            self.icon16_renderer,
            self.icon32_renderer
        ]

        # Image factories
        for image_factory_id in self.image_factories:
            key = image_factory_id + "_image_factory"
            method_name = "create_%s_image_factory" % image_factory_id
            method = getattr(self, method_name)
            image_factory = method()
            setattr(self, key, image_factory)
            self.configuration.image_factories.append(image_factory)

        # Extensions
        self.enable_extensions()

    def create_configuration(self):
        return self._create(
            Configuration,
            qname = "woost.configuration",
            secret_key = random_string(10),
            default_language = self.languages[0],
            languages = self.languages,
            backoffice_language =
                first(
                    language
                    for language in self.languages
                    if language in
                        Configuration.backoffice_language.enumeration
                )
        )

    def create_administrator(self):
        return self._create(
            User,
            qname = "woost.administrator",
            email = self.admin_email,
            password = self.admin_password
        )

    def create_website(self):
        return self._create(
            Website,
            site_name = TranslatedValues("website.site_name"),
            hosts = self.hosts
        )

    def create_anonymous_role(self):
        return self._create(
            Role,
            qname = "woost.anonymous",
            implicit = True,
            title = TranslatedValues()
        )

    def create_anonymous_user(self):
        return self._create(
            User,
            qname = "woost.anonymous_user",
            email = "anonymous@localhost",
            roles = [self.anonymous_role],
            anonymous = True
        )

    def create_administrator_role(self):
        return self._create(
            Role,
            qname = "woost.administrators",
            title = TranslatedValues(),
            permissions = [
                self._create(ReadPermission, content_type = Item),
                self._create(CreatePermission, content_type = Item),
                self._create(ModifyPermission, content_type = Item),
                self._create(DeletePermission, content_type = Item),
                self._create(ReadMemberPermission),
                self._create(ModifyMemberPermission),
                self._create(InstallationSyncPermission)
            ]
        )

    def create_everybody_role(self):
        role = self._create(
            Role,
            implicit = True,
            qname = "woost.everybody",
            title = TranslatedValues(),
            permissions = [
                self._create(RenderPermission, content_type = Item),
                self._create(
                    ReadPermission,
                    content_type = Publishable,
                    content_expression =
                        "from woost.models import user_has_access_level\n"
                        "items.add_filter(user_has_access_level)"
                )
            ]
        )

        # Restrict readable members
        if self.restricted_members:
            role.permissions.append(
                self._create(
                    ReadMemberPermission,
                    authorized = False,
                    matching_members = self.restricted_members
                )
            )

        role.permissions.append(self._create(ReadMemberPermission))

        # Restrict modifiable members
        if self.read_only_members:
            role.permissions.append(
                self._create(
                    ModifyMemberPermission,
                    authorized = False,
                    matching_members = self.read_only_members
                )
            )

        role.permissions.append(self._create(ModifyMemberPermission))

        # All languages allowed
        role.permissions.extend([
            self._create(CreateTranslationPermission),
            self._create(ReadTranslationPermission),
            self._create(ModifyTranslationPermission),
            self._create(DeleteTranslationPermission)
        ])

        return role

    def create_authenticated_role(self):
        return self._create(
            Role,
            qname = "woost.authenticated",
            implicit = True,
            title = TranslatedValues()
        )

    def create_editor_role(self):
        role = self._create(
            Role,
            qname = "woost.editors",
            title = TranslatedValues(),
            hidden_content_types = [
                Template,
                Controller,
                Style,
                EmailTemplate
            ]
        )

        # Restrict readable types
        if self.restricted_types:
            for restricted_type in self.restricted_types:
                for permission_type in (
                    ReadPermission,
                    CreatePermission,
                    ModifyPermission,
                    DeletePermission
                ):
                    role.permissions.append(
                        self._create(
                            permission_type,
                            authorized = False,
                            content_type = restricted_type
                        )
                    )

        # Restrict modifiable types
        if self.read_only_types:
            for read_only_type in self.read_only_types:
                for permission_type in (
                    CreatePermission,
                    ModifyPermission,
                    DeletePermission
                ):
                    role.permissions.append(
                        self._create(
                            permission_type,
                            authorized = False,
                            content_type = read_only_type
                        )
                    )

        for permission_type in (
            ReadPermission,
            CreatePermission,
            ModifyPermission,
            DeletePermission,
            ReadHistoryPermission
        ):
            role.permissions.append(
                self._create(
                    permission_type,
                    content_type = Item
                )
            )

        role.permissions.append(
            self._create(
                InstallationSyncPermission,
                authorized = False
            )
        )

        return role

    def create_editor_access_level(self):
        return self._create(
            AccessLevel,
            qname = "woost.editor_access_level",
            roles_with_access = [self.editor_role]
        )

    def create_file_deletion_trigger(self):
        return self._create(
            DeleteTrigger,
            qname = "woost.file_deletion_trigger",
            title = TranslatedValues(),
            execution_point = "after",
            batch_execution = True,
            content_type = File,
            responses = [
                self._create(
                    CustomTriggerResponse,
                    code = u"from os import remove\n"
                           u"for item in items:\n"
                           u"    remove(item.file_path)"
                )
            ]
        )

    def create_controllers(self):
        for controller_name in (
            "Document",
            "File",
            "URI",
            "Styles",
            "Feed",
            "BackOffice",
            "FirstChildRedirection",
            "Login",
            "PasswordChange",
            "PasswordChangeConfirmation"
        ):
            controller = self._create(
                Controller,
                qname = "woost.%s_controller" % controller_name.lower(),
                title = TranslatedValues(),
                python_name = "woost.controllers.%scontroller.%sController" % (
                    controller_name.lower(),
                    controller_name
                )
            )
            setattr(self, controller_name.lower() + "_controller", controller)

        # The backoffice controller is placed at an irregular location
        self.backoffice_controller.python_name = (
            "woost.controllers.backoffice.backofficecontroller."
            "BackOfficeController"
        )

        # Prevent anonymous access to the backoffice controller
        self._create(
            ReadPermission,
            role = self.anonymous_role,
            content_type = Publishable,
            content_expression =
                """items.add_filter(cls.qname.equal("woost.backoffice"))""",
            authorized = False
        )

    def create_standard_template(self):
        return self._create(
            Template,
            qname = "woost.standard_template",
            title = TranslatedValues(),
            identifier = "woost.views.GenericSiteLayout",
        )

    def create_home(self):
        return self._create(
            Page,
            title = TranslatedValues("home.title"),
            inner_title = TranslatedValues("home.inner_title"),
            blocks = [
                self._create(
                    TextBlock,
                    text = TranslatedValues("home.body")
                )
            ]
        )

    def create_user_stylesheet(self):
        return self._create(
            Document,
            qname = "woost.user_styles",
            title = TranslatedValues(),
            per_language_publication = False,
            controller = self.styles_controller,
            path = "user_styles",
            hidden = True,
            mime_type = "text/css",
            caching_policies = [
                self._create(
                    CachingPolicy,
                    cache_tags_expression =
                        "tags.add('woost.models.style.Style')\n",
                    server_side_cache = True
                )
            ]
        )

    def create_backoffice(self):
        return self._create(
            Document,
            qname = "woost.backoffice",
            title = TranslatedValues(),
            hidden = True,
            path = "cms",
            per_language_publication = False,
            controller = self.backoffice_controller
        )

    def create_not_found_error_page(self):
        return self._create(
            Page,
            qname = "woost.not_found_error_page",
            title = TranslatedValues(),
            hidden = True,
            blocks = [
                self._create(
                    TextBlock,
                    text = TranslatedValues("not_found_error_page.body")
                )
            ]
        )

    def create_forbidden_error_page(self):
        return self._create(
            Page,
            qname = "woost.forbidden_error_page",
            title = TranslatedValues(),
            hidden = True,
            blocks = [
                self._create(
                    TextBlock,
                    text = TranslatedValues("forbidden_error_page.body")
                )
            ]
        )

    def create_password_change_page(self):
        return self._create(
            Page,
            qname = "woost.password_change_page",
            title = TranslatedValues(),
            controller = self.passwordchange_controller,
            hidden = True,
            blocks = [
                self._create(
                    TextBlock,
                    text = TranslatedValues("password_change_page.body")
                ),
                self._create(
                    CustomBlock,
                    heading = TranslatedValues("password_change_page.form_title"),
                    view_class = "woost.views.PasswordChangeRequestForm",
                    controller =
                        "woost.controllers.passwordchangecontroller."
                        "PasswordChangeBlockController"
                )
            ]
        )

    def create_password_change_confirmation_page(self):
        return self._create(
            Page,
            qname = "woost.password_change_confirmation_page",
            title = TranslatedValues(),
            controller = self.passwordchangeconfirmation_controller,
            hidden = True,
            blocks = [
                self._create(
                    TextBlock,
                    text = TranslatedValues("password_change_confirmation_page.body")
                ),
                self._create(
                    CustomBlock,
                    heading = TranslatedValues(
                        "password_change_confirmation_page.form_title"
                    ),
                    view_class = "woost.views.PasswordChangeConfirmationForm",
                    controller =
                        "woost.controllers.passwordchangecontroller."
                        "PasswordChangeConfirmationBlockController"
                )
            ]
        )

    def create_password_change_email_template(self):
        return self._create(
            EmailTemplate,
            qname = "woost.password_change_email_template",
            title = TranslatedValues(),
            subject = TranslatedValues(),
            body = TranslatedValues(),
            sender = u'u"noreply@' + self.hosts[0] + '"',
            receivers = u"[user.email]"
        )

    def create_login_page(self):
        return self._create(
            Page,
            qname = "woost.login_page",
            title = TranslatedValues(),
            hidden = True,
            controller = self.login_controller,
            blocks = [
                self._create(LoginBlock)
            ]
        )

    def create_page_tree_user_view(self):
        return self._create(
            UserView,
            qname = "woost.page_tree_user_view",
            title = TranslatedValues(),
            roles = [self.everybody_role],
            parameters = {
                "type": "woost.models.publishable.Publishable",
                "content_view": "tree",
                "filter": None,
                "members": None
            }
        )

    def create_file_gallery_user_view(self):
        return self._create(
            UserView,
            qname = "woost.file_gallery_user_view",
            title = TranslatedValues(),
            roles = [self.everybody_role],
            parameters = {
                "type": "woost.models.file.File",
                "content_view": "thumbnails",
                "filter": None,
                "order": None,
                "members": None
            }
        )

    def create_content_renderer(self):
        return self._create(
            rendering.ChainRenderer,
            qname = "woost.content_renderer",
            title = TranslatedValues(),
            renderers = [
                self._create(rendering.ImageFileRenderer),
                self._create(rendering.PDFRenderer),
                self._create(rendering.VideoFileRenderer),
                self._create(rendering.ImageURIRenderer)
            ]
        )

    def create_icon16_renderer(self):
        return self._create(
            rendering.IconRenderer,
            qname = "woost.icon16_renderer",
            title = TranslatedValues(),
            icon_size = 16
        )

    def create_icon32_renderer(self):
        return self._create(
            rendering.IconRenderer,
            qname = "woost.icon32_renderer",
            title = TranslatedValues(),
            icon_size = 32
        )

    def create_default_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.default_image_factory",
            title = TranslatedValues(),
            identifier = "default"
        )

    def create_icon16_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.icon16_image_factory",
            title = TranslatedValues(),
            identifier = "icon16",
            renderer = self.icon16_renderer,
            applicable_to_blocks = False
        )

    def create_icon32_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.icon32_image_factory",
            title = TranslatedValues(),
            identifier = "icon32",
            renderer = self.icon32_renderer,
            applicable_to_blocks = False
        )

    def create_backoffice_thumbnail_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.backoffice_thumbnail_image_factory",
            title = TranslatedValues(),
            identifier = "backoffice_thumbnail",
            effects = [
                self._create(
                    rendering.Thumbnail,
                    width = "100",
                    height = "100"
                ),
                self._create(
                    rendering.Frame,
                    edge_width = 1,
                    edge_color = "ddd",
                    vertical_padding = "4",
                    horizontal_padding = "4",
                    background = "eee"
                )
            ],
            fallback = self.icon32_image_factory,
            applicable_to_blocks = False
        )

    def create_backoffice_small_thumbnail_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.backoffice_small_thumbnail_image_factory",
            title = TranslatedValues(),
            identifier = "backoffice_small_thumbnail",
            effects = [
                self._create(
                    rendering.Thumbnail,
                    width = "32",
                    height = "32"
                )
            ],
            fallback = self.icon16_image_factory,
            applicable_to_blocks = False
        )

    def create_edit_blocks_thumbnail_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.edit_blocks_thumbnail_image_factory",
            title = TranslatedValues(),
            identifier = "edit_blocks_thumbnail",
            effects = [
                self._create(
                    rendering.Thumbnail,
                    width = "75",
                    height = "75"
                ),
                self._create(
                    rendering.Frame,
                    edge_width = 1,
                    edge_color = "ccc",
                    vertical_padding = "4",
                    horizontal_padding = "4",
                    background = "eee"
                )
            ],
            applicable_to_blocks = False
        )

    def create_close_up_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.close_up_image_factory",
            title = TranslatedValues(),
            identifier = "image_gallery_close_up",
            effects = [
                self._create(
                    rendering.Fill,
                    width = "900",
                    height = "700",
                    preserve_vertical_images = True
                )
            ]
        )

    def create_default_thumbnail_image_factory(self):
        return self._create(
            rendering.ImageFactory,
            qname = "woost.default_thumbnail_image_factory",
            title = TranslatedValues(),
            identifier = "image_gallery_thumbnail",
            effects = [
                self._create(
                    rendering.Fill,
                    width = "200",
                    height = "150",
                    preserve_vertical_images = True
                )
            ]
        )

    def enable_extensions(self):
        # Enable the selected extensions
        if self.extensions:
            load_extensions()
            for extension in Extension.select():
                ext_name = extension.__class__.__name__[:-len("Extension")].lower()
                if ext_name in self.extensions:
                    extension.enabled = True




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import webbrowser
from cocktail.translations import *
from cocktail.persistence import *
from woost import app
from woost.models import *
from woost.models.utils import *
from woost.models.extension import load_extensions

def setup_shell(env):
    env["config"] = config = Configuration.instance
    config.setup_languages()
    set_current_user(User.require_instance(qname = "woost.anonymous_user"))
    set_current_website(config.websites[0])
    load_extensions()

get = Item.get_instance
req = Item.require_instance

def show(target):

    if isinstance(target, Publishable):
        webbrowser.open(target.get_uri(host = "!"))
    elif hasattr(target, "__iter__"):
        for item in target:
            show(item)
    else:
        print "Can't show non-publishable object %r" % target

def edit(target):

    backoffice = Publishable.require_instance(qname = "woost.backoffice")

    if isinstance(target, Item):
        webbrowser.open(
            backoffice.get_uri(
                host = "!",
                path = ["content", str(target.id)]
            )
        )
    elif hasattr(target, "__iter__"):
        for item in target:
            edit(item)
    else:
        print "Can't edit object %r" % target

